/**
 * PicoLCD 4x20 
 * Copyright (C) 2008 mini-box.com
 *
 * Licensed under the terms of the GNU Lesser General Public License v2.1:
 * 		http://www.opensource.org/licenses/lgpl-2.1.php
 *
 * For further information visit:
 * 		http://www.picolcd.com/
 *
 * File Authors:
 * 		Barna Csenteri
 */


#include "stdafx.h"
#include "HIDPico4x20.h"
#include "HIDDevice.h"
#include "HIDDescriptor.h"
#include "HIDGraphics.h"
#include "HEXFile.h"
#include "utils\TxFile.h"
#include "HIDDevices.h"
#include <math.h>

#define A_SLEEP			5      
#define A_TIMEOUT		3000
#define BUF_LINE_LEN	47
#define STR_LINE_LEN	57
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HIDPico4x20::HIDPico4x20(int type, HIDDevice* hdev, int timeout):HIDInterface(hdev)
{
	if (hdev)
		MyTrace(0, "HID device %s %s opened (type=%d)\n"
		, hdev->GetDescriptor()->m_sManufacturer
		, hdev->GetDescriptor()->m_sProduct
		, type );
	else MyTrace(0, "HID device could not be opened (type=%d)\n", type);

	m_nType				= type;
		
	m_nError			= -1;
	m_bWriteOK			= true;
	m_pGraphics			= NULL;
	m_nVersion[0]		= -1;
	m_nVersion[1]		= -1;

	m_nSplashMaxSize	= 0;
	m_nSplashSize		= 0;

	m_nTimeout = timeout;
	
	int cnt = 0;
	do{
		if (!isFlasher())
		{
			if (m_nType == HIDDevices::DEV_TYPE_LCD)
			{
#ifdef TESTCODE4graphlcd
					 //test code for the graph interface
					HArray ha1;x
					ha1.setHIDPkt(0x93, 2, 2, 0x01, 0x00);
					m_pParent->Write(&ha1);

					for (int i=0;i<4;i++)
					{
						unsigned char cs = ((i << 2) & 0xFF);
						HArray ha2;
						ha2.setHIDPkt(OUT_REPORT_CMD, 8, 8, cs, 0x02, 0x00, 0x64, 0x3F, 0x00, 0x64, 0xC0);
						m_pParent->Write(&ha2);
					}

					HArray ha;
					ha.setHIDPkt(OUT_REPORT_LCD_CONTRAST, 1, 1, 0xe8);//0x40-status);
					m_pParent->Write(&ha);

					/*for (unsigned char i=0;i<4;i++)
					{
						unsigned char cs = (i << 2);//chipselect

						//{
						//HArray ha2;
						//ha2.setHIDPkt(OUT_REPORT_CMD, 5, 5, cs, 0x01, 0x00, 0x00, 0x3E);
						//m_pParent->Write(&ha2);
						//}

						for (unsigned char j=0;j<8;j++)
						{
							HArray ha2;
							ha2.setHIDPkt(OUT_REPORT_CMD, 8, 8, cs, 0x02, 0x00, 0x00, 0xb8|j, 0x00, 0x00, 0x40);
							m_pParent->Write(&ha2);

							HArray ha3;
							ha3.setHIDPkt(OUT_REPORT_DATA, 63, 4, cs | 0x01, 0x00, 0x00, 32);
							
							for (int x=0;x<32;x++)
								ha3.getBuf()[5+x] = 0xFF;

							m_pParent->Write_NoBuf(&ha3);
							m_pParent->Write_NoBuf(&ha3);//send twice for 64 byte
						}

						//{
						//HArray ha2;
						//ha2.setHIDPkt(OUT_REPORT_CMD, 5, 5, cs, 0x01, 0x00, 0x00, 0x3F);
						//m_pParent->Write(&ha2);
						//}
					}*/
					
					for (unsigned char j=0;j<8;j++)
					{
						for (unsigned char i=0;i<4;i++)
						{
							HArray ha2;
							ha2.setHIDPkt(OUT_REPORT_CMD, 8, 8, i << 2, 0x02, 0x00, 0x00, 0xb8|j, 0x00, 0x00, 0x40);
							m_pParent->Write(&ha2);
						}
						
						for (unsigned char i=0;i<4;i++)						
						{
							unsigned char cs = (i << 2);//chipselect

							HArray ha3;
							ha3.setHIDPkt(OUT_REPORT_DATA, 63, 4, cs | 0x01, 0x00, 0x00, 32);
							
							for (int x=0;x<32;x++)
								ha3.getBuf()[5+x] = 0xF0|j;

							m_pParent->Write_NoBuf(&ha3);
							m_pParent->Write_NoBuf(&ha3);//send twice for 64 byte
						}
					}

					/*for (unsigned char i=0;i<4;i++)
					{
						HArray ha2;
						ha2.setHIDPkt(OUT_REPORT_CMD, 8, 8, i << 2, 0x02, 0x00, 0x00, 0xb8|7, 0x00, 0x00, 0x40);
						m_pParent->Write(&ha2);
					}

					for (unsigned char j=0;j<8;j++)
					{
						for (unsigned char i=0;i<4;i++)						
						{
							unsigned char cs = (i << 2);//chipselect

							HArray ha3;
							ha3.setHIDPkt(OUT_REPORT_DATA, 63, 4, cs | 0x01, 0x00, 0x00, 32);
							
							for (int x=0;x<32;x++)
								ha3.getBuf()[5+x] = 0xF0+j;

							m_pParent->Write_NoBuf(&ha3);
							m_pParent->Write_NoBuf(&ha3);//send twice for 64 byte
						}

						if (j<7)
						{
							for (unsigned char i=0;i<4;i++)						
							{
								HArray ha2;
								ha2.setHIDPkt(OUT_REPORT_CMD, 8, 8, i << 2, 0x02, 0x00, 0x00, 0xc0|(8*(j+1)), 0x00, 0x00, 0xb8|j);
								m_pParent->Write(&ha2);
							}
						}
					}*/

					for (unsigned char i=0;i<4;i++)
					{
						unsigned char cs = (i << 2);//chipselect
						for (unsigned char j=0;j<8;j++)
						{
							HArray ha2;
							ha2.setHIDPkt(OUT_REPORT_CMD, 8, 8, cs, 0x02, 0x00, 0x00, 0xb8|j, 0x00, 0x00, 0x40);
							m_pParent->Write(&ha2);

							HArray ha3;
							ha3.setHIDPkt(OUT_REPORT_DATA, 4+32, 4, cs | 0x01, 0x00, 0x00, 32);
							
							for (int x=0;x<32;x++)
								ha3.getBuf()[5+x] = 0x00;

							m_pParent->Write(&ha3);
							m_pParent->Write(&ha3);//send twice for 64 byte
						}
					}
#endif

				TxFile file("lcd.ini");
				if (file.Open() != 0)
				{//ini file sequence not found - write internal one
					HArray ha;
					ha.setHIDPkt(OUT_REPORT_CMD, 23, 23, 0x00, 0x07, 0x00, 0x32, 0x30, 0x00, 0x32, 0x30, 0x00, 0x32, 0x30, 0x00, 0x32, 0x38, 0x00, 0x32, 0x06, 0x00, 0x32, 0x0C, 0x07, 0xD0, 0x01);
					m_bWriteOK = m_pParent->Write(&ha);
				}
				else
				{
					HArray txarray;
					int ret = file.NextLine(&txarray);
					while (ret != -1)
					{
						if (ret == 1)
						{
							if (txarray.length()>0)
							{
								if ((txarray.getBuf()[0] == OUT_REPORT_CMD)||(txarray.getBuf()[0] == OUT_REPORT_DATA))
								{
									m_bWriteOK = m_pParent->Write(&txarray);
								}
							}
						}
						ret = file.NextLine(&txarray);
					}
				}

				getVer();
				//Sleep(1000);
				getMaxSplash();
				//Sleep(1000);

				setLedStatus(0);
				setBacklight(0x14);
				setContrast(0xff);

#ifdef _LCD_4x20
				setText(0,0, "    USB-LCD-4x20    ");
				setText(0,1, "  www.mini-box.com  ");
#endif
			}
			else if (m_nType == HIDDevices::DEV_TYPE_INSTALL)
			{
				getVer();
			}
			else if (m_nType == HIDDevices::DEV_TYPE_IR)
			{
				//do nothing since this is the infrared device
			}

		}
		else
		{
			getVer();
			getVer();
			getVer();
			getVer();
			getVer();
		}

		if (!m_bWriteOK)
		{
			CloseDevice();
			Sleep(1000);
			OpenDevice();
		}
		cnt++;
	}while ((!m_bWriteOK)&&(cnt<5));
}

HIDPico4x20::~HIDPico4x20()
{
	m_nError = 0;
}

void HIDPico4x20::Received(HArray* array)
{
	m_aReceived.setValue(array);

	switch (array->getBuf()[0])
	{
	case IN_REPORT_KEY_STATE:
		{
			MyTrace(3, ">>> KP %x %x\n", array->getBuf()[1], array->getBuf()[2]);

			if (m_pGraphics)
				m_pGraphics->UpdateStatus(false, STATUS_KEY, array->getBuf()[1], array->getBuf()[2]);
		}break;
	case IN_REPORT_IR_DATA:
		{
			MyTrace(3, ">>> IR (%02d)", array->getBuf()[1]);
			int i = 0;
			for (i=0;i<array->getBuf()[1];i++)
				MyTrace(2, " %x", array->getBuf()[2+i]);
			MyTrace(2, "\n");

			char buf[256], bt[5];
			sprintf(buf, "notify ir %d",array->getBuf()[1]);
			for (i=0;i<array->getBuf()[1];i++)
			{
				sprintf(bt," x%02x", array->getBuf()[2+i]);
				strcat(buf, bt);
			}

			if (m_pGraphics)
				m_pGraphics->UpdateStatus(false, STATUS_IR, array->getBuf()[1], array->getBuf()[2]);
			
		}break;
	case RESULT_PARAMETER_MISSING:
		{
			MyTrace(2, "Parameter missing\n");
		}break;
	case RESULT_DATA_MISSING:
		{
			MyTrace(2, "Data missing\n");
		}break;
	case RESULT_BLOCK_READ_ONLY:
		{
			MyTrace(2, "Block readonly\n");
		}break;
	case RESULT_BLOCK_TOO_BIG:
		{
			MyTrace(2, "Block too big\n");
		}break;
	case RESULT_SECTION_OVERFLOW:
		{
			MyTrace(2, "Section overflow\n");
		}break;
	case HID_REPORT_GET_VERSION_1:
		{
			MyTrace(2, "HID_REPORT_GET_VERSION_1\n");
			m_nVersion[0] = array->getBuf()[2];
			m_nVersion[1] = array->getBuf()[1];
			if (m_pGraphics)
				m_pGraphics->UpdateStatus(false, STATUS_VER, 0 , 0);
			m_nError = 0;
		}break;
	case HID_REPORT_GET_VERSION_2:
		{
			MyTrace(2, "HID_REPORT_GET_VERSION_2\n");
			m_nVersion[0] = array->getBuf()[2];
			m_nVersion[1] = array->getBuf()[1];
			if (m_pGraphics)
				m_pGraphics->UpdateStatus(false, STATUS_VER, 0 , 0);
			m_nError = 0;
		}break;
	case HID_REPORT_GET_MAX_STX_SIZE:
		{
			MyTrace(2, "HID_REPORT_GET_MAX_STX_SIZE ");

			memcpy(&m_nSplashMaxSize, array->getBuf()+1, 2);
			memcpy(&m_nSplashSize, array->getBuf()+3, 2);

			MyTrace(2, "max=%d size=%d\n",m_nSplashMaxSize, m_nSplashSize);

			m_nError = 0;
		}break;
	case FLASH_REPORT_ERASE_MEMORY:
	case KEYBD_REPORT_ERASE_MEMORY:
		{
			MyTrace(2, "XXXX_REPORT_ERASE_MEMORY\n");
			m_nError = 0;
		}break;
	case FLASH_REPORT_READ_MEMORY:
	case KEYBD_REPORT_READ_MEMORY:
		{
			MyTrace(2, "XXXX_REPORT_READ_MEMORY\n");
			m_nError = 0;
		}break;
	case FLASH_REPORT_WRITE_MEMORY:
	case KEYBD_REPORT_WRITE_MEMORY:
		{
			MyTrace(2, "XXXX_REPORT_WRITE_MEMORY\n");
			m_nError = 0;
		}break;
	case IN_REPORT_EXT_EE_DATA:
		{
			MyTrace(2, "IN_REPORT_EXT_EE_DATA\n");
			m_nError = 0;
		}break;
	case IN_REPORT_INT_EE_DATA:
		{
			MyTrace(2, "IN_REPORT_INT_EE_DATA\n");
			m_nError = 0;
		}break;
	case KEYBD_REPORT_MEMORY:
		{
			MyTrace(2, "KEYBD_REPORT_MEMORY\n");
			m_nError = 0;
		}break;
	case OUT_REPORT_RELAY_ONOFF:
		{
			MyTrace(2, "OUT_REPORT_RELAY_ONOFF\n");
			m_nError = 0;
		}break;
/*	case OUT_REPORT_EXT_EE_READ:
		{
			MyTrace(2, "OUT_REPORT_EXT_EE_READ\n");
			m_nError = 0;
		}break;
	case OUT_REPORT_EXT_EE_WRITE:
		{
			MyTrace(2, "OUT_REPORT_EXT_EE_WRITE\n");
			m_nError = 0;
		}break;
	case OUT_REPORT_INT_EE_READ:
		{
			MyTrace(2, "OUT_REPORT_INT_EE_READ\n");
			m_nError = 0;
		}break;
	case OUT_REPORT_INT_EE_WRITE:
		{
			MyTrace(2, "OUT_REPORT_INT_EE_WRITE\n");
			m_nError = 0;
		}break;*/
	case HID_REPORT_EXIT_FLASHER://ff is errorcode as well in this case
		{
			MyTrace(2, "HID_REPORT_EXIT_FLASHER\n");
			if (array->length() > 1)
				 m_nError = array->getBuf()[1];
			else m_nError = 2;
		}break;
	case HID_REPORT_EXIT_KEYBOARD://ff is errorcode as well in this case
		{
			MyTrace(2, "HID_REPORT_EXIT_KEYBOARD\n");
			if (array->length() > 1)
				 m_nError = array->getBuf()[1];
			else m_nError = 2;
		}break;
	case HID_ERROR:
		{
			MyTrace(2, "HID_ERROR\n");
			if (array->length() > 2)
				 m_nError = (array->getBuf()[2] | 0xFFFF0000);
			else m_nError = 2;
		}break;
	default:
		{
			MyTrace(0, "Unknown income %x\n", array->getBuf()[0]);
			m_nError = 1;
		}
	}
}

void HIDPico4x20::DeviceDisconnected()
{
	m_nError = -1;
	if (m_pGraphics)
		m_pGraphics->UpdateStatus(false, STATUS_DISCONNECTED, 0, 0);
}

void HIDPico4x20::getVer()
{
	if (!m_pParent) return;

	HArray ha;
	if (m_nType == HIDDevices::DEV_TYPE_INSTALL)
		 ha.setHIDPkt(HID_REPORT_GET_VERSION_2, 0, 0);
	else ha.setHIDPkt(HID_REPORT_GET_VERSION_1, 0, 0);
	m_bWriteOK = m_pParent->Write(&ha);
}

void HIDPico4x20::getMaxSplash()
{
	if (!m_pParent) return;

	HArray ha;
	ha.setHIDPkt(HID_REPORT_GET_MAX_STX_SIZE, 0, 0);
	m_bWriteOK = m_pParent->Write(&ha);

	m_nError = -1;
	
	int tmout = 0;
	while (m_nError == -1)
	{
		Sleep(A_SLEEP);
		tmout++;
		if (tmout > A_TIMEOUT) return;
	}
}

void HIDPico4x20::setLedStatus(unsigned char status)
{
	m_cLedStatus = status;

	if (!m_pParent) return;

	HArray ha;
	ha.setHIDPkt(OUT_REPORT_LED_STATE, 1, 1, status);
	m_bWriteOK = m_pParent->Write(&ha);
}

void HIDPico4x20::setBacklight(unsigned char status)
{
	m_cBacklight = status;

	if (!m_pParent) return;

	HArray ha;
	ha.setHIDPkt(OUT_REPORT_LCD_BACKLIGHT, 1, 1, status);
	m_bWriteOK = m_pParent->Write(&ha);
}

void HIDPico4x20::setContrast(unsigned char status)
{
	TRACE("CONRAST: %x\n", status);

	m_cContrast = status;

	if (!m_pParent) return;

	HArray ha;
	ha.setHIDPkt(OUT_REPORT_LCD_CONTRAST, 1, 1, status);
	m_bWriteOK = m_pParent->Write(&ha);
}

#ifdef _LCD_4x20
void HIDPico4x20::setText(const char x, const char y, const char* text, unsigned int len)
{
	if (len == 0) len = strlen(text);
	if (len > LINE_LENGTH) len = LINE_LENGTH;

	int nx = x;
	char* line;

	int ny = y;
	if (y>=LINE_NO) 
		ny = LINE_NO;

	line = m_cLine[ny];

	HArray hapos(false);
	hapos.setHIDPkt(OUT_REPORT_CMD, 5, 4, 0x00, 0x01, 0x00, 0x64);
	switch (ny)
	{
	case 1:hapos.getBuf()[5] = 0xc0;break;
	case 2:hapos.getBuf()[5] = 0x94;break;
	case 3:hapos.getBuf()[5] = 0xd4;break;
	case 0:
	default:
		hapos.getBuf()[5] = 0x80;break;
	}
	
	if (m_pParent) 
		 m_bWriteOK = m_pParent->Write(&hapos);
	else m_bWriteOK = false;

	HArray ha(false);
	ha.setHIDPkt(OUT_REPORT_DATA, 4+len, 4, 0x01,0x00, 0x01, (unsigned char)len);

	for (unsigned int i=0;i<len;i++)
	{
		ha.getBuf()[5+i] = text[i];
		
		if (nx < LINE_LENGTH) line[nx++] = text[i];
	}

	char buf[LINE_LENGTH*2+1];
	GetLine(buf, y);

	if (m_pParent) 
		 m_bWriteOK = m_pParent->Write(&ha);
	else m_bWriteOK = false;
}
#endif
void HIDPico4x20::GetLine(char* buf, int ln)
{
	char* line = m_cLine[ln];
	int cnt_buf = 0;
	for (unsigned int n=0;n<LINE_LENGTH;n++)
	{
		switch (line[n])
		{
		case '\\'	:buf[cnt_buf++] = '\\';buf[cnt_buf++] = '\\';break;
		case 0		:buf[cnt_buf++] = '\\';buf[cnt_buf++] = '0';break;
		case 1		:buf[cnt_buf++] = '\\';buf[cnt_buf++] = '1';break;
		case 2		:buf[cnt_buf++] = '\\';buf[cnt_buf++] = '2';break;
		case 3		:buf[cnt_buf++] = '\\';buf[cnt_buf++] = '3';break;
		case 4		:buf[cnt_buf++] = '\\';buf[cnt_buf++] = '4';break;
		case 5		:buf[cnt_buf++] = '\\';buf[cnt_buf++] = '5';break;
		case 6		:buf[cnt_buf++] = '\\';buf[cnt_buf++] = '6';break;
		case 7		:buf[cnt_buf++] = '\\';buf[cnt_buf++] = '7';break;
		case 8		:buf[cnt_buf++] = '\\';buf[cnt_buf++] = '8';break;
		case 9		:buf[cnt_buf++] = '\\';buf[cnt_buf++] = '9';break;
		default		:buf[cnt_buf++] = line[n];
		}
		buf[cnt_buf] = 0;
	}
}

void HIDPico4x20::getinitmesgs(char* buf)
{
	char buf1[LINE_LENGTH*2+1],buf2[LINE_LENGTH*2+1];
	GetLine(buf1, 0);
	GetLine(buf2, 1);

	sprintf(buf, "set led x%02x\r\nset lcd x%02x\r\nset blight %d\r\nset contrast %d\r\nset text 0 0 %s\r\nset text 0 1 %s"
		,m_cLedStatus,m_cLCDStatus,m_cBacklight,m_cContrast,buf1,buf2);
	
}

#ifdef _LCD_4x20
void HIDPico4x20::setFont(const char n, const char map[8])
{
	if (!m_pParent) return;

	HArray ha1;
	ha1.setHIDPkt(OUT_REPORT_CMD , 5, 5, 0x00, 0x01, 0x00, 0x64, 0x40+8*n);
	m_bWriteOK = m_pParent->Write(&ha1);

	HArray ha2;
	ha2.setHIDPkt(OUT_REPORT_DATA, 12, 12, 0x01, 0x00, 0x01, 0x08, map[0], map[1], map[2], map[3], map[4], map[5], map[6], map[7]);
	m_bWriteOK = m_pParent->Write(&ha2);
}
#endif
void HIDPico4x20::testSplash()
{
	if (!m_pParent) return;

	HArray ha;
	ha.setHIDPkt(OUT_REPORT_TESTSPLASH, 1, 1, 0xff);
	m_bWriteOK = m_pParent->Write(&ha);
}

bool HIDPico4x20::isFlasher()
{
	if (!m_pParent) return false;
	return (m_pParent->GetDescriptor()->m_nPID == ID_PROD_FLASH);
}

void HIDPico4x20::switchMode()
{
	MyTrace(0, "Switching mode, tout=%d(%x)\n",m_nTimeout,m_nTimeout);

//	HArray ha1;
//	ha1.setHIDPkt(HID_SET_SNOOZE_TIME, 2, 2, m_nTimeout & 0xff, (m_nTimeout >> 8) & 0xff);
//	m_bWriteOK = m_pParent->Write(&ha1);

	m_nError = -1;
	m_nVersion[0] = -1;
	m_nVersion[1] = -1;

	if (isFlasher())
	{
		HArray ha;
		ha.setHIDPkt(HID_REPORT_EXIT_FLASHER, 2, 2, m_nTimeout & 0xff, (m_nTimeout >> 8) & 0xff);
		m_bWriteOK = m_pParent->Write(&ha);
	}
	else
	{
		HArray ha;
		ha.setHIDPkt(HID_REPORT_EXIT_KEYBOARD, 2, 2, m_nTimeout & 0xff, (m_nTimeout >> 8) & 0xff);
		m_bWriteOK = m_pParent->Write(&ha);
	}

	int tmout = 0;
	while (m_nError == -1)
	{
		Sleep(A_SLEEP);
		tmout++;
		if (tmout > A_TIMEOUT) return;
	}
}

bool HIDPico4x20::_isEnabled(int type)
{
	switch (type)
	{
	case TYPE_CODE_MEMORY:
		{
			if (isFlasher()) return true;
		}break;
	case TYPE_EPROM_EXTERNAL:
		{
			if (!isFlasher()) return true;
		}break;
	case TYPE_EPROM_INTERNAL:
		{
			if (!isFlasher()) return true;
		}break;
	case TYPE_CODE_SPLASH:
		{
			if (!isFlasher()) return true;
		}break;
	}
	return false;
}

int HIDPico4x20::_getWriteMsg(int type)
{
	switch (type)
	{
	case TYPE_CODE_MEMORY:		return FLASH_REPORT_WRITE_MEMORY;
	case TYPE_EPROM_EXTERNAL:	return OUT_REPORT_EXT_EE_WRITE;
	case TYPE_EPROM_INTERNAL:	return OUT_REPORT_INT_EE_WRITE;
	case TYPE_CODE_SPLASH:		return KEYBD_REPORT_WRITE_MEMORY;
	}
	return 0;
}

int HIDPico4x20::_getReadMsg(int type)
{
	switch (type)
	{
	case TYPE_CODE_MEMORY:		return FLASH_REPORT_READ_MEMORY;
	case TYPE_EPROM_EXTERNAL:	return OUT_REPORT_EXT_EE_READ;
	case TYPE_EPROM_INTERNAL:	return OUT_REPORT_INT_EE_READ;
	case TYPE_CODE_SPLASH:		return KEYBD_REPORT_READ_MEMORY;
	}
	return 0;
}

bool HIDPico4x20::writeFile(bool fromsocket, int type, const char* file, const unsigned long end)
{
	if (!_isEnabled(type)) return false;

	m_lockCommand.lock();
	int n = _writeFile(fromsocket, type, file, end);
	if (n != 0)
	{
		if (type == TYPE_CODE_MEMORY)
		{
			MyTrace(0, "Error occured writing file: %d > ERASING address x2000\n", n);
			
			unsigned long addr = 0x2000;

			HArray erase;
			erase.setHIDPkt(FLASH_REPORT_ERASE_MEMORY, 3, 3, addr & 0xff, (addr >> 8) & 0xff, (addr >> 16) & 0xff);
			m_bWriteOK = m_pParent->Write(&erase);
		}
		else if (type == TYPE_CODE_SPLASH)
		{
			MyTrace(0, "Error occured writing file: %d > ERASING address x0000\n", n);
			
			//unsigned long addr = 0x0000;

			//HArray erase;
			//erase.setHIDPkt(KEYBD_REPORT_ERASE_MEMORY, 2, 2, addr & 0xff, (addr >> 8) & 0xff);
			//m_bWriteOK = m_pParent->Write(&erase);
		}
		else MyTrace(0, "Error occured writing file: %d (type=%d)\n", n, type);

		if (m_pGraphics)
			m_pGraphics->UpdateStatus(fromsocket, STATUS_ERROR, STATUS_WRITE, n);
	}
	else
	{
		MyTrace(0, "Writing file succesfull: %d (type=%d)\n", n, type);

		if (m_pGraphics)
			m_pGraphics->UpdateStatus(fromsocket, STATUS_OK, STATUS_WRITE, 0);
	}
	m_lockCommand.unlock();

	return (n==0);
}

int HIDPico4x20::_writeFile(bool fromsocket, int type, const char* file, const unsigned long end)
{
	int tmout = 0;
	HEXFile hfile(file);
	if (hfile.Open() == 0)
	{

		unsigned long start_addr, stop_addr, last_addr = 0;
		unsigned long length = 0;
		unsigned long last_length = 0;
		int ret;

		if ((type == TYPE_CODE_MEMORY)||(type == TYPE_CODE_SPLASH))
		{
			int lenght_statistics = 0;
			ret = hfile.NextLine();
			int start_statistics = hfile.GetGlobalAddress();
			while (ret == 0)
			{
				lenght_statistics = hfile.GetGlobalAddress()+hfile.GetLength();
				ret = hfile.NextLine();
			}

			hfile.Reset();
			ret = hfile.NextLine();
			
			start_addr = hfile.GetGlobalAddress();
			length = hfile.GetLength();

			while ((ret==0)||(ret==1))
			{
				if (ret == 0)
				{
					stop_addr = hfile.GetGlobalAddress();

					if (end != -1)
					{
						if (stop_addr >= end)
						{
							MyTrace(1, "erase: end reached 1(%x)\n",stop_addr);
							ret = -1000;
						}
						else
						{
							if ((stop_addr + 64) >= end)
							{
								MyTrace(1, "erase: end reached 2(%x > %x)\n",stop_addr,end - 64);
								ret = -1000;
							}
						}
					}
				}

				if (  ((stop_addr-last_addr) > 64)
					||(ret == -1000)
					||(ret == 1)//end of file
					)
				{
					if ((ret == -1000)||(ret == 1))
					{
						last_addr = stop_addr;
						last_length = length;
					}
					//delete last continuous data

					MyTrace(1, ">>> Deleting mem pack: %x - %x\n", start_addr, last_addr+last_length);
					int del_addr;
					for (del_addr=start_addr;del_addr<=(last_addr+last_length);del_addr+=64)
					{
						if (m_pGraphics)
							m_pGraphics->UpdateStatus(fromsocket, STATUS_ERASE, (lenght_statistics-start_statistics)/64, (del_addr-start_statistics)/64);

						m_nError = -1;

						HArray erase;
						if (type == TYPE_CODE_MEMORY)
							 erase.setHIDPkt(FLASH_REPORT_ERASE_MEMORY, 3, 3, del_addr & 0xff, (del_addr >> 8) & 0xff, (del_addr >> 16) & 0xff);
						else erase.setHIDPkt(KEYBD_REPORT_ERASE_MEMORY, 2, 2, del_addr & 0xff, (del_addr >> 8) & 0xff);
						m_bWriteOK = m_pParent->Write(&erase);

						tmout = 0;
						while (m_nError == -1)
						{
							Sleep(A_SLEEP);
							tmout++;
							if (tmout > A_TIMEOUT) return -1;
						}

						if (m_nError != 0) return m_nError;

						MyTrace(1, "ERASE1: addr: %x (%d) len=%d \n" ,del_addr, del_addr, hfile.GetLength());
					}
					
					//delete the last address for 100% match
					del_addr = last_addr + last_length;

					m_nError = -1;
					HArray erase;
					if (type == TYPE_CODE_MEMORY)
						 erase.setHIDPkt(FLASH_REPORT_ERASE_MEMORY, 3, 3, del_addr & 0xff, (del_addr >> 8) & 0xff, (del_addr >> 16) & 0xff);
					else erase.setHIDPkt(KEYBD_REPORT_ERASE_MEMORY, 2, 2, del_addr & 0xff, (del_addr >> 8) & 0xff);
					m_bWriteOK = m_pParent->Write(&erase);

					tmout = 0;
					while (m_nError == -1)
					{
						Sleep(A_SLEEP);
						tmout++;
						if (tmout > A_TIMEOUT) return -1;
					}

					if (m_nError != 0) return m_nError;

					MyTrace(1, "ERASE2: addr: %x (%d) len=%d \n" ,del_addr, del_addr, hfile.GetLength());


					start_addr = stop_addr;
				}

				last_addr = stop_addr;
				last_length = length;
				
				if ((ret != -1000)&&(ret!=1))
				{//last line reached end
					ret = hfile.NextLine();
					if (ret > 1) return -2;//wrong hex
				}
				else if (ret == 1)
					ret = -1000;//end the file cycle completly
			}
		}

		int addr;
		hfile.Reset();
		ret = hfile.NextLine();
		int len;
		while (ret==0)
		{
			if (m_pGraphics)
				m_pGraphics->UpdateStatus(fromsocket, STATUS_WRITE, hfile.GetMaxLines(), hfile.GetCurLine()+1);

			addr = hfile.GetGlobalAddress();
			len  = hfile.GetLength();

			if (end != -1)
			{
				if (addr >= end)
				{
					MyTrace(0, "write: jump1: %x \n",addr);
					break;
				}

				if ((addr + len) >= end)
				{
					len = end - addr;
					ret = -1000;

					MyTrace(0, "write: jump2: %x (len=%d) \n",addr, len);
				}
			}

			m_nError = -1;
			
			HArray ha;

			if (type == TYPE_CODE_MEMORY)
			{
				ha.setHIDPkt(_getWriteMsg(type), 4+len, 4, addr & 0xff, (addr >> 8) & 0xff, (addr >> 16) & 0xff, len);
	 			for (int i=0;i<len;i++)
					ha.getBuf()[5+i] = hfile.GetData()[i];
			}
			else
			{
				ha.setHIDPkt(_getWriteMsg(type), 3+len, 3, addr & 0xff, (addr >> 8) & 0xff, len);
				for (int i=0;i<len;i++)
					ha.getBuf()[4+i] = hfile.GetData()[i];
			}

			MyTrace(1, "write: addr: %x (%d)\n",addr, len);
				
			m_bWriteOK = m_pParent->Write(&ha);
			tmout = 0;
			while (m_nError == -1)
			{
				Sleep(A_SLEEP);
				tmout++;
				if (tmout > A_TIMEOUT) return -14;
			}
			
			if (m_nError != 0) return m_nError;

			if (type == TYPE_CODE_MEMORY)
			{
				if (!ha.equals(&m_aReceived, 4+len))
					return -3;
			}
			else
			{
				if (!ha.equalsSpecial(&m_aReceived, 3+len))
					return -3;
			}

			if (ret != -1000)
			{
				ret = hfile.NextLine();
				if (ret > 1) return -15;
			}
		}
	}
	else
	{
		MyTrace(1,"Invalid hex file!\n");

		return -100;
	}
	
	return 0;
}

void HIDPico4x20::eraseFlash(bool fromsocket, const unsigned long start, const unsigned long end)
{
	if (!isFlasher()) return;

	m_lockCommand.lock();
	int n = _eraseFlash(fromsocket, start, end);
	if (n != 0)
	{
		MyTrace(0, "Error occured erasing: %d\n", n);
		
		if (m_pGraphics)
			m_pGraphics->UpdateStatus(fromsocket, STATUS_ERROR, STATUS_ERASE, n);
	}
	else
	{
		MyTrace(0, "Erasing succesfull: %d\n", n);

		if (m_pGraphics)
			m_pGraphics->UpdateStatus(fromsocket, STATUS_OK, STATUS_ERASE, 0);
	}
	m_lockCommand.unlock();
}

int HIDPico4x20::_eraseFlash(bool fromsocket, const unsigned long start, const unsigned long end)
{
	if (end<=start) return -100;

	unsigned long lines = (end - start) / 64;

	int tmout;
	unsigned long addr = start;
	while (addr < end)
	{
		if (m_pGraphics)
			m_pGraphics->UpdateStatus(fromsocket, STATUS_ERASE, lines, ((addr - start) / 64) + 1);

		m_nError = -1;

		HArray erase;
		if (isFlasher())
			 erase.setHIDPkt(FLASH_REPORT_ERASE_MEMORY, 3, 3, addr & 0xff, (addr >> 8) & 0xff, (addr >> 16) & 0xff);
		else erase.setHIDPkt(KEYBD_REPORT_ERASE_MEMORY, 2, 2, addr & 0xff, (addr >> 8) & 0xff);
		m_bWriteOK = m_pParent->Write(&erase);

		tmout = 0;
		while (m_nError == -1)
		{
			Sleep(A_SLEEP);
			tmout++;
			if (tmout > A_TIMEOUT) return -1;
		}

		if (m_nError != 0) return m_nError;

		addr += 64;
	}

	return 0;
}

bool HIDPico4x20::readFile(bool fromsocket, int type, const char* file,const unsigned long start, const unsigned long end)
{
	//if (!_isEnabled(type)) return false;//flash read permitted in both modes

	m_lockCommand.lock();
	int n = _readFile(fromsocket, type, file, start, end);
	if (n != 0)
	{
		MyTrace(0, "Error occured reading: %d (type=%d)\n", n, type);
		
		if (m_pGraphics)
			m_pGraphics->UpdateStatus(fromsocket, STATUS_ERROR, STATUS_READ, n);
	}
	else
	{
		MyTrace(0, "Reading succesfull: %d (type=%d)\n", n, type);

		if (m_pGraphics)
			m_pGraphics->UpdateStatus(fromsocket, STATUS_OK, STATUS_READ, 0);
	}
	m_lockCommand.unlock();

	return (n==0);
}

int HIDPico4x20::_readFile(bool fromsocket, int type, const char* file,const unsigned long start, const unsigned long end)
{
	if (end<=start) return -100;

	int online_len = PKG_LEN_FLASH-5;//32
	if (type != TYPE_CODE_MEMORY) online_len = PKG_LEN_LCD-4;//20

	unsigned long lines = ceil( (double)(end - start) / (double)online_len );//32

	FILE* fw = fopen(file, "wt");
	if (!fw) return -1;

	fprintf(fw, ":020000040000FA\r\n");

	int tmout;
	unsigned long addr = start, oldaddr = start;
	int temp;
	while (addr < end)
	{
		if (m_pGraphics)
			m_pGraphics->UpdateStatus(fromsocket, STATUS_READ, lines, ((addr - start) / online_len) + 1);

		m_nError = -1;

		HArray read;
		if (type == TYPE_CODE_MEMORY)
		{
			temp = end - addr;
			if (temp>online_len) temp=online_len;//32
			read.setHIDPkt(_getReadMsg(type), 4, 4, addr & 0xff, (addr >> 8) & 0xff, (addr >> 16) & 0xff, temp);
		}
		else
		{
			temp = end - addr;
			if (temp>online_len) temp=online_len;//20
			read.setHIDPkt(_getReadMsg(type), 3, 3, addr & 0xff, (addr >> 8) & 0xff, temp);
		}

		m_bWriteOK = m_pParent->Write(&read);

		tmout = 0;
		while (m_nError == -1)
		{
			Sleep(A_SLEEP);
			tmout++;
			if (tmout > A_TIMEOUT)
			{
				fprintf(fw, "ERROR: timeout\r\n");
				fclose(fw);
				return -2;
			}
		}

		if (m_nError != 0)
		{
			fprintf(fw, "ERROR: device responded with %x\r\n", m_nError);
			fclose(fw);
			return m_nError;
		}
/*
TX[37]:F3 A0 23 00 20 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 0
0 00 00 00 00 00 00 00 00 00 00 00 00
RX[37]:F3 A0 23 00 20 FF FF FF FF FF FF FF FF FF FF FF FF FF FF FF FF FF FF FF F
F FF FF FF FF FF FF FF FF FF FF FF FF
*/

		unsigned char checksum = 0;
		if (type == TYPE_CODE_MEMORY)
		{
			fprintf(fw, ":%02X",m_aReceived.getAt(4));
			checksum += m_aReceived.getAt(4);
			fprintf(fw, "%02X",((addr >> 8) & 0xff));
			checksum += ((addr >> 8) & 0xff);
			fprintf(fw, "%02X",addr & 0xff);
			checksum += (addr & 0xff);
			fprintf(fw, "00");
			for (unsigned int i=5;i<(m_aReceived.getAt(4)+5);i++)
			{
				fprintf(fw, "%02X",m_aReceived.getAt(i));
				checksum += m_aReceived.getAt(i);
			}
			checksum = 0x01 + ~checksum;
			fprintf(fw, "%02X\r\n",checksum);

			addr += temp;//32
		}
		else
		{
			fprintf(fw, ":%02X",m_aReceived.getAt(3));
			checksum += m_aReceived.getAt(3);
			fprintf(fw, "%02X",((addr >> 8) & 0xff));
			checksum += ((addr >> 8) & 0xff);
			fprintf(fw, "%02X",addr & 0xff);
			checksum += (addr & 0xff);
			fprintf(fw, "00");
			for (unsigned int i=4;i<(m_aReceived.getAt(3)+4);i++)
			{
				fprintf(fw, "%02X",m_aReceived.getAt(i));
				checksum += m_aReceived.getAt(i);
			}
			checksum = 0x01 + ~checksum;
			fprintf(fw, "%02X\r\n",checksum);	
			
			addr += temp;//20;
		}
	}

	fprintf(fw,":00000001FF");
	fclose(fw);
	return 0;
}

bool HIDPico4x20::writeFile4Array(int line_len, const char* file, const unsigned char* buffer, const unsigned long length)
{
	FILE* fw = fopen(file, "wt");
	if (!fw) return false;

	fprintf(fw, ":020000040000FA\r\n");
	unsigned long addr = 0;
	unsigned char checksum = 0;
	int len;
	while (addr < length)
	{
		checksum = 0;
		
		len = line_len;
		if ((addr + line_len) > length) len = length - addr;


		fprintf(fw, ":%02X",len);
		checksum += len;
		fprintf(fw, "%02X",((addr >> 8) & 0xff));
		checksum += ((addr >> 8) & 0xff);
		fprintf(fw, "%02X",addr & 0xff);
		checksum += (addr & 0xff);
		fprintf(fw, "00");

		for (unsigned long i=addr;i<(addr+len);i++)
		{
			fprintf(fw, "%02X",(unsigned char)buffer[i]);
			checksum += buffer[i];
		}

		checksum = 0x01 + ~checksum;
		fprintf(fw, "%02X\r\n",checksum);

		addr+=line_len;
	}

	fprintf(fw,":00000001FF");

	fclose(fw);

	return true;
}

void HIDPico4x20::sendCmd(const char* command, bool from_socket)
{
	if (!command) return;
	int len = strlen(command);
	if (!len) return;

	m_lockCommand.lock();

	char* tcmd = new char[len+1];
	strcpy(tcmd, command);
	char* cmd = strtok(tcmd, " \r\n");
	char* sub = strtok(NULL, " \r\n");
	
	bool ok = false;
	if (strcmp(cmd, "set") == 0)
	{
		if (isFlasher())
		{
			;
		}
		else if (strcmp(sub, "led") == 0)
		{
			char* status = strtok(NULL, " \r\n");
			if (status)
			{
				unsigned char nstat = (unsigned char)getno(status);

				setLedStatus(nstat);

				ok = true;
			}
		}
		else if (strcmp(sub, "blight") == 0)
		{
			char* status = strtok(NULL, " \r\n");
			if (status)
			{
				unsigned char nstat = (unsigned char)getno(status);

				setBacklight(nstat);

				ok = true;
			}
		}
		else if (strcmp(sub, "contrast") == 0)
		{
			char* status = strtok(NULL, " \r\n");
			if (status)
			{
				unsigned char nstat = (unsigned char)getno(status);

				setContrast(nstat);

				ok = true;
			}
		}
#ifdef _LCD_4x20
		else if (strcmp(sub, "text") == 0)
		{
			char* x = strtok(NULL, " \r\n");
			char* y = strtok(NULL, " \r\n");
			char* text = strtok(NULL, "");
		    if (x && y && text)
			{
				char nx = (char)getno(x);
				if (nx < 0) nx = 0;
				if (nx > (LINE_LENGTH-1)) nx = LINE_LENGTH-1;

				char ny = (char)getno(y);
				if (ny < 0) ny = 0;
				if (ny > 1) ny = 1;

				int tl = convertslash(text);
				setText(nx, ny, text, tl);

				ok = true;
			}
		}
		else if (strcmp(sub, "font") == 0)
		{
			char* no = strtok(NULL, " \r\n");
			char* chars[8];
			chars[0] = strtok(NULL, " \r\n");
			chars[1] = strtok(NULL, " \r\n");
			chars[2] = strtok(NULL, " \r\n");
			chars[3] = strtok(NULL, " \r\n");
			chars[4] = strtok(NULL, " \r\n");
			chars[5] = strtok(NULL, " \r\n");
			chars[6] = strtok(NULL, " \r\n");
			chars[7] = strtok(NULL, " \r\n");

			if (no && chars[0] && chars[1] && chars[2] && chars[3] && chars[4] && chars[5] && chars[6] && chars[7])
			{
				char n = (char) getno(no);
				char c[8];
				c[0] = (char) getno(chars[0]);
				c[1] = (char) getno(chars[1]);
				c[2] = (char) getno(chars[2]);
				c[3] = (char) getno(chars[3]);
				c[4] = (char) getno(chars[4]);
				c[5] = (char) getno(chars[5]);
				c[6] = (char) getno(chars[6]);
				c[7] = (char) getno(chars[7]);

				setFont(n, c);

				ok = true;
			}
		}
#endif
	}//set
	else if (strcmp(cmd, "get") == 0)
	{
		if (isFlasher())
		{
			;
		}
	}
	
	m_lockCommand.unlock();

	if (!ok)
	{
		MyTrace(0, "ERROR: (Unknown command: %s)\n", command);
		if (!from_socket)
		{
			MyTrace(0, "Available commands:\n");
			MyTrace(0, " set led xff/12\n");
			MyTrace(0, " set blight 0/1\n");
			MyTrace(0, " set contrast 0/1\n");
			MyTrace(0, " set clear\n");
			MyTrace(0, " set text x y text\n");
			MyTrace(0, " set font no f0 f1 f2 f3 f4 f5 f6 f7\n");
			MyTrace(0, " set relay msec\n");
			MyTrace(0, " set splash\n");
			MyTrace(0, " set inte min|sec|nextslot|repetition|leds|line1(20chars)|line2(20chars)|min|sec|...\n");
			MyTrace(0, " set exte min|sec|nextslot|repetition|leds|line1(20chars)|line2(20chars)|min|sec|...\n");
			MyTrace(0, " get inte\n");
			MyTrace(0, " get exte\n");
		}
	}

	delete tcmd;
}

void HIDPico4x20::connectGraph(HIDGraphics* hg)
{
	m_lockCommand.lock();
	m_pGraphics = hg;
	m_lockCommand.unlock();
}

bool HIDPico4x20::loadFile2Array(unsigned char* data, const unsigned long len, const char* file)
{
	memset(data, 0xff, len);

	HEXFile hfile(file);
	if (hfile.Open() == 0)
	{
		unsigned long addr,oaddr = -1;
		int ret = hfile.NextLine();
		while (ret==0)
		{
			addr = hfile.GetGlobalAddress();
			for (unsigned long p = 0;p<hfile.GetLength();p++)
			{
				if ((addr+p)<len)
				{
					data[addr+p] = hfile.GetData()[p];
				}
				else 
					return true;
			}

			ret = hfile.NextLine();
			if (ret > 1)
			{
				return false;
			}
		}
	}
	else
	{
		MyTrace(1,"Invalid hex file!\n");

		return false;
	}

	return true;
}
