/**
 * PicoLCD 4x20 
 * Copyright (C) 2008 mini-box.com
 *
 * Licensed under the terms of the GNU Lesser General Public License v2.1:
 * 		http://www.opensource.org/licenses/lgpl-2.1.php
 *
 * For further information visit:
 * 		http://www.picolcd.com/
 *
 * File Authors:
 * 		Barna Csenteri
 */


#if !defined(AFX_HIDPico4x20_H__297E4426_06E1_4E28_B062_EECB4C88DF5C__INCLUDED_)
#define AFX_HIDPico4x20_H__297E4426_06E1_4E28_B062_EECB4C88DF5C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "HIDInterface.h"
#include "utils/HLock.h"

#define LINE_NO			4
#define LINE_LENGTH		20
#define MAX_BUF			500

class HIDSocket;
class HIDGraphics;

#define STATUS_OK				0x00
#define STATUS_ERASE			0x01
#define STATUS_WRITE			0x02
#define STATUS_READ				0x03
#define STATUS_ERROR			0xff
#define STATUS_KEY				0x10
#define STATUS_IR				0x11
#define STATUS_VER				0x12
#define STATUS_DISCONNECTED		0x13

#define HID_ERROR					0x10

//Keyboard mode
#define IN_REPORT_POWER_STATE		0x01
#define IN_REPORT_KEY_STATE			0x11
#define IN_REPORT_IR_DATA			0x21
#define OUT_REPORT_LED_STATE		0x81
#define OUT_REPORT_LCD_BACKLIGHT	0x91
#define OUT_REPORT_LCD_CONTRAST		0x92

#define OUT_REPORT_CMD				0x94
#define OUT_REPORT_DATA				0x95

//flasher mode
#define RESULT_OK					0x00
#define RESULT_PARAMETER_MISSING	0x01            	// Request does not contain all necessary parameters (request is too short, no length or no address either).
#define RESULT_DATA_MISSING			0x02				// Request attempts to write more data than was actually sent.
#define RESULT_BLOCK_READ_ONLY		0x03				// Request attempts to erase or write a block within the flasher code section.
#define RESULT_BLOCK_NOT_ERASABLE	0x04            	// Request attempts to erase a block outside of "erasable" memory space.
#define RESULT_BLOCK_TOO_BIG 		0x05          		// Request attempts to access more data at once than the HID buffer can handle.
#define RESULT_SECTION_OVERFLOW		0x06				// Request (address & length) attempts to cross over a section boundary (section boundaries are set at 1Mbyte boundaries -> 0x100000, 0x200000... etc.).
#define HID_REPORT_GET_VERSION_1	0xF1				// No parameters
#define HID_REPORT_GET_VERSION_2	0xF7				// No parameters

#define FLASH_REPORT_ERASE_MEMORY	0xF2				// AddressLo : AddressHi : AddressUp (anywhere inside the 64 byte-block to be erased)
#define FLASH_REPORT_READ_MEMORY	0xF3				// AddressLo : AddressHi : AddressUp : Data Length (1...32)
#define FLASH_REPORT_WRITE_MEMORY	0xF4				// AddressLo : AddressHi : AddressUp : Data Length (1...32) : Data....
#define KEYBD_REPORT_ERASE_MEMORY	0xB2				// same as F2 but in keyboard mode
#define KEYBD_REPORT_READ_MEMORY	0xB3				// same as F3 but in keyboard mode
#define KEYBD_REPORT_WRITE_MEMORY	0xB4				// same as F4 but in keyboard mode
#define KEYBD_REPORT_MEMORY			0x41				// response to b3,b4

#define HID_REPORT_GET_MAX_STX_SIZE	0xF6

#define HID_REPORT_EXIT_FLASHER		0xFF				// No parameters
#define HID_REPORT_EXIT_KEYBOARD	0xEF				// No parameters
#define HID_SET_SNOOZE_TIME			0xF8				// 0-65535 (1234h > 34 12);

#define OUT_REPORT_RELAY_ONOFF		0xB1				// 0-65535 (1234h > 34 12); - internal relay on/off
#define OUT_REPORT_TESTSPLASH		0xC1				// test splash screen

#define IN_REPORT_EXT_EE_DATA		0x31
#define OUT_REPORT_EXT_EE_READ		0xA1
#define OUT_REPORT_EXT_EE_WRITE		0xA2

#define IN_REPORT_INT_EE_DATA		0x32
#define OUT_REPORT_INT_EE_READ		0xA3
#define OUT_REPORT_INT_EE_WRITE		0xA4

#define ID_SPLASH_TIMER				0x71
#define ID_SPLASH_CYCLE_START		0x72
#define ID_SPLASH_CYCLE_END			0x73

#define TYPE_CODE_MEMORY			0x00
#define TYPE_EPROM_EXTERNAL			0x01
#define TYPE_EPROM_INTERNAL			0x02
#define TYPE_CODE_SPLASH			0x03

class HIDPico4x20 : public HIDInterface
{
public:
	HIDPico4x20(int type, HIDDevice* hdev, int timeout);
	virtual ~HIDPico4x20();

	void getinitmesgs(char* buf);//return's all init messages

	void setLedStatus(unsigned char status);
	void setBacklight(unsigned char status);
	void setContrast(unsigned char status);
#ifdef _LCD_4x20
	void setText(const char x, const char y, const char* text, unsigned int len = 0);
	void setFont(const char n, const char map[8]);
#endif

	void testSplash();

	void sendCmd(const char* command, bool from_socket);
	void connectGraph(HIDGraphics* hg);

	void getVer();
	void getMaxSplash();

	/** Commands */
	bool writeFile(bool fromsocket, int type, const char* file, const unsigned long end);
	void eraseFlash(bool fromsocket, const unsigned long start, const unsigned long end);
	bool readFile(bool fromsocket, int type, const char* file,const unsigned long start, const unsigned long end);
	
	void switchMode();
	static bool loadFile2Array(unsigned char* data, const unsigned long len, const char* file);

	/** Write file from array*/
	bool writeFile4Array(int data_len, const char* file, const unsigned char* buffer, const unsigned long length);

	bool isFlasher();

	bool isWOK(){return m_bWriteOK;};

	int getSplashMaxSize(){if (m_nSplashMaxSize>64) return m_nSplashMaxSize-64;else return m_nSplashMaxSize;};
	int getSplashSize(){return m_nSplashSize;};

	virtual void Received(HArray* array);
	virtual void DeviceDisconnected();

	int m_nVersion[2];
private:
	int m_nType;

	void GetLine(char* buf, int ln);
	
	int _eraseFlash(bool fromsocket, const unsigned long start, const unsigned long end);
	int _writeFile(bool fromsocket, int type, const char* file, const unsigned long end);
	int _readFile(bool fromsocket, int type, const char* file,const unsigned long start, const unsigned long end);

	bool _isEnabled(int type);
	int _getWriteMsg(int type);
	int _getReadMsg(int type);

	unsigned char m_cLedStatus;
	unsigned char m_cLCDStatus;
	unsigned char m_cBacklight;
	unsigned char m_cContrast;
	char m_cLine[LINE_NO][LINE_LENGTH];

	unsigned short m_nSplashMaxSize;
	unsigned short m_nSplashSize;

	unsigned char m_cBuff[MAX_BUF];

	int m_nTimeout;

	int  m_nError;
	bool m_bWriteOK;
	HArray m_aReceived;

	HLock m_lockSock;
	HLock m_lockCommand;
	
	HIDGraphics* m_pGraphics;
};

#endif // !defined(AFX_HIDPico4x20_H__297E4426_06E1_4E28_B062_EECB4C88DF5C__INCLUDED_)
