/**
 * PicoLCD 4x20 
 * Copyright (C) 2008 mini-box.com
 *
 * Licensed under the terms of the GNU Lesser General Public License v2.1:
 * 		http://www.opensource.org/licenses/lgpl-2.1.php
 *
 * For further information visit:
 * 		http://www.picolcd.com/
 *
 * File Authors:
 * 		Barna Csenteri
 */


#include "stdafx.h"
#include "PicoLCD4x20.h"
#include "CardLCD.h"
#include "PicoLCD4x20Dlg.h"
#include "..\SplashEditor\PopupCommand.h"

// CCardLCD dialog

IMPLEMENT_DYNAMIC(CCardLCD, CPropertyPage)

CCardLCD::CCardLCD()
	: CPropertyPage(CCardLCD::IDD)
{
	m_nTimer = 0;
	m_strLCD = "";
}

CCardLCD::~CCardLCD()
{
	DestroyWindow();
}

void CCardLCD::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_EDIT_TEXT, m_editText);
	DDX_Control(pDX, IDC_CHECK_LED1, m_cboxLed[0]);
	DDX_Control(pDX, IDC_CHECK_LED2, m_cboxLed[1]);
	DDX_Control(pDX, IDC_CHECK_LED3, m_cboxLed[2]);
	DDX_Control(pDX, IDC_CHECK_LED4, m_cboxLed[3]);
	DDX_Control(pDX, IDC_CHECK_LED5, m_cboxLed[4]);
	DDX_Control(pDX, IDC_CHECK_LED6, m_cboxLed[5]);
	DDX_Control(pDX, IDC_CHECK_LED7, m_cboxLed[6]);
	DDX_Control(pDX, IDC_CHECK_LED8, m_cboxLed[7]);
	DDX_Control(pDX, IDC_SLIDER_CONTRAST, m_sliderContrast);
	DDX_Control(pDX, IDC_SLIDER_BRIGHTNESS, m_sliderBacklight);
	DDX_Control(pDX, IDC_STATIC_KEY1, m_staticKey[0]);
	DDX_Control(pDX, IDC_STATIC_KEY2, m_staticKey[1]);
	DDX_Control(pDX, IDC_STATIC_KEY3, m_staticKey[2]);
	DDX_Control(pDX, IDC_STATIC_KEY4, m_staticKey[3]);
	DDX_Control(pDX, IDC_STATIC_KEY5, m_staticKey[4]);
	DDX_Control(pDX, IDC_STATIC_KEY6, m_staticKey[5]);
	DDX_Control(pDX, IDC_STATIC_KEY7, m_staticKey[6]);
	DDX_Control(pDX, IDC_STATIC_KEY8, m_staticKey[7]);
	DDX_Control(pDX, IDC_PROGRESS_IR, m_progressIR);
}


BEGIN_MESSAGE_MAP(CCardLCD, CPropertyPage)
	ON_BN_CLICKED(IDC_CHECK_LED1, &CCardLCD::OnBnClickedCheckLed1)
	ON_BN_CLICKED(IDC_CHECK_LED2, &CCardLCD::OnBnClickedCheckLed2)
	ON_BN_CLICKED(IDC_CHECK_LED3, &CCardLCD::OnBnClickedCheckLed3)
	ON_BN_CLICKED(IDC_CHECK_LED4, &CCardLCD::OnBnClickedCheckLed4)
	ON_BN_CLICKED(IDC_CHECK_LED5, &CCardLCD::OnBnClickedCheckLed5)
	ON_BN_CLICKED(IDC_CHECK_LED6, &CCardLCD::OnBnClickedCheckLed6)
	ON_BN_CLICKED(IDC_CHECK_LED7, &CCardLCD::OnBnClickedCheckLed7)
	ON_BN_CLICKED(IDC_CHECK_LED8, &CCardLCD::OnBnClickedCheckLed8)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_SLIDER_CONTRAST, &CCardLCD::OnNMReleasedcaptureSliderContrast)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_SLIDER_BRIGHTNESS, &CCardLCD::OnNMReleasedcaptureSliderBrightness)
	ON_WM_TIMER()
	ON_WM_SHOWWINDOW()
	ON_BN_CLICKED(IDC_BUTTON_COMMANDS, &CCardLCD::OnBnClickedButtonCommands)
END_MESSAGE_MAP()


// CCardLCD message handlers
void CCardLCD::SendInitialValues()
{
	SendText();
	OnBnClickedCheckLed1();
	SendContrast();
	SendBacklight();
	UpdateKeys(0,0);
}

void CCardLCD::SendContrast()
{
//	TRACE("Contrast: %d\n",m_sliderContrast.GetPos());
	if (((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD)
		((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->setContrast(m_sliderContrast.GetPos());
}

void CCardLCD::SendBacklight()
{
//	TRACE("Backlight: %d\n",0x14-m_sliderBacklight.GetPos());
	if (((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD)
		((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->setBacklight(0x14-m_sliderBacklight.GetPos());
}

void CCardLCD::UpdateKeys(int n1, int n2)
{
	m_nKeys[0] = n1;
	m_nKeys[1] = n2;

	bool keys[8];
	for (int i=0;i<8;i++)
		keys[i] = false;

	if (m_nKeys[0] != 0)
		keys[ m_nKeys[0] - 1 ] = true;

	if (m_nKeys[1] != 0)
		keys[ m_nKeys[1] - 1 ] = true;

	for (int i=0;i<8;i++)
	{
		if (keys[i])
			 m_staticKey[i].ShowWindow(SW_SHOW);
		else m_staticKey[i].ShowWindow(SW_HIDE);
	}
}

void CCardLCD::UpdateIR()
{
	m_progressIR.SetPos(1);
}

void CCardLCD::SendText()
{
	CString str;
	m_editText.GetWindowText(str);
	
	m_strLCD = str;

	char buf[LINE_LENGTH*LINE_NO*4];

	checkEnter(buf, str);

#ifdef _LCD_4x20
	if (((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD)
	{
		for (int line=0;line<LINE_NO;line++)
			((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->setText(0, line, buf+line*LINE_LENGTH, LINE_LENGTH);
	}
#endif
}

BOOL CCardLCD::OnInitDialog()
{
	CPropertyPage::OnInitDialog();

	SetControlParent(this,IDC_EDIT_TEXT);
	SetControlParent(this,IDC_SLIDER_CONTRAST);
	SetControlParent(this,IDC_SLIDER_BRIGHTNESS);
	SetControlParent(this,IDC_PROGRESS_IR);
	SetControlParent(this,IDC_CHECK_LED1);
	SetControlParent(this,IDC_CHECK_LED2);
	SetControlParent(this,IDC_CHECK_LED3);
	SetControlParent(this,IDC_CHECK_LED4);
	SetControlParent(this,IDC_CHECK_LED5);
	SetControlParent(this,IDC_CHECK_LED6);
	SetControlParent(this,IDC_CHECK_LED7);
	SetControlParent(this,IDC_CHECK_LED8);
	SetControlParent(this,IDC_BUTTON_COMMANDS);

	m_editText.SetFont(&((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_fontBig);
	m_editText.SetWindowText("    USB-LCD-4x20\r\n  www.mini-box.com\r\n Test tool started\r\nSpec. chars:\\0\\1\\2\\3\\4\\5\\6\\7");

	m_sliderContrast.SetRange(0, 0xFF);
//	m_sliderContrast.SetPos(0xFF);
	m_sliderBacklight.SetRange(0, 0x14);
//	m_sliderBacklight.SetPos(0x14);

	for (int i=5;i<8;i++) 
		m_cboxLed[i].SetCheck(1);

	m_progressIR.SetRange(0,1);

	m_nTimer = SetTimer(1, 500, NULL);

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CCardLCD::OnBnClickedCheckLed1()
{
	unsigned char led = 0;
	for (int i=0;i<8;i++)
	{
		if (m_cboxLed[i].GetCheck() != 0)
			led |= (1<<i);
	}
	
	if (((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD)
		((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->setLedStatus(led);
}

void CCardLCD::OnBnClickedCheckLed2()
{
	OnBnClickedCheckLed1();
}

void CCardLCD::OnBnClickedCheckLed3()
{
	OnBnClickedCheckLed1();
}

void CCardLCD::OnBnClickedCheckLed4()
{
	OnBnClickedCheckLed1();
}

void CCardLCD::OnBnClickedCheckLed5()
{
	OnBnClickedCheckLed1();
}

void CCardLCD::OnBnClickedCheckLed6()
{
	OnBnClickedCheckLed1();
}

void CCardLCD::OnBnClickedCheckLed7()
{
	OnBnClickedCheckLed1();
}

void CCardLCD::OnBnClickedCheckLed8()
{
	OnBnClickedCheckLed1();
}
void CCardLCD::OnNMReleasedcaptureSliderContrast(NMHDR *pNMHDR, LRESULT *pResult)
{
	SendContrast();
	*pResult = 0;
}

void CCardLCD::OnNMReleasedcaptureSliderBrightness(NMHDR *pNMHDR, LRESULT *pResult)
{
	SendBacklight();
	*pResult = 0;
}

BOOL CCardLCD::DestroyWindow()
{
	if (m_nTimer!=0) KillTimer(m_nTimer);

	return CPropertyPage::DestroyWindow();
}

void CCardLCD::OnTimer(UINT_PTR nIDEvent)
{
	if (GetSafeHwnd() && IsWindowVisible())
	{
		//TRACE("!!!!!!! Visible!\n");

		if (m_progressIR.GetSafeHwnd())
		{
			if (m_progressIR.GetPos()!=0)
				m_progressIR.SetPos(0);
		}

		if (m_editText.GetSafeHwnd())
		{
			CString str;
			m_editText.GetWindowText(str);

			if (str != m_strLCD) 
			{
				if  (   (((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD)
					&& (!(((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->isFlasher()))
					)
					SendText();
			}
		}
	}
	//else TRACE("!!!!!!! Invisible!\n");

	CPropertyPage::OnTimer(nIDEvent);
}

void CCardLCD::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CPropertyPage::OnShowWindow(bShow, nStatus);

	if (bShow && GetSafeHwnd())
		SendInitialValues();

	/*if (((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterface)
	{
		if ( ((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterface->isFlasher() )
		{
			AfxMessageBox("Disabled in flasher mode!");
		}
	}*/
	/*if (bShow && GetSafeHwnd())
	{
		CRect rect;
		GetWindowPlacement(&rect);

		rect.right  = rect.left  + 500;
		rect.bottom = rect.top	 + 400;
		
		AfxGetMainWnd()->MoveWindow(rect);
	}*/
}

void CCardLCD::OnBnClickedButtonCommands()
{
	CPopupCommand dlg(this);
	if (dlg.DoModal() == IDOK)
	{
		Str_Command str;
		str.nType  = SPLASH_COMMAND;
		str.nIOMap = 0x00;

		dlg.RetrieveCommand(&str);
		
		HArray ha(false);
		ha.setHIDPkt(OUT_REPORT_CMD, 2+3*str.nLen, 2, str.nIOMap , str.nLen);

		for (int i=0;i<str.nLen;i++)
		{
			ha.getBuf()[3+i*3] = (str.nTime[i] >> 8) & 0xFF;
			ha.getBuf()[4+i*3] = str.nTime[i] & 0xFF;
			ha.getBuf()[5+i*3] = str.nValue[i];
		}

		if  (((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD )
		{
			((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->WriteArray(&ha);

			if (dlg.m_bClearDisplay)
			{
				m_strLCD = "";
				m_editText.SetWindowText("");
			}
		}

	}
}
