#include "stdafx.h"
#include "ListTypes.h"

int GetSplashTypeLen(int type)
{
	switch (type)
	{
	case SPLASH_BRIGHTNESS:		return sizeof(Str_Brightness);
	case SPLASH_CONTRAST:		return sizeof(Str_Contrast);
	case SPLASH_TEXT:			return sizeof(Str_Text);
	case SPLASH_TIMER:			return sizeof(Str_Timer);
	case SPLASH_COMMAND:		return sizeof(Str_Command);
	case SPLASH_LED:			return sizeof(Str_Led);
	case SPLASH_FONT:			return sizeof(Str_Font);
	case SPLASH_REPEAT_START:	return sizeof(Str_Repeat_Start);
	case SPLASH_REPEAT_STOP:	return sizeof(Str_Repeat_Stop);
	}
	return -1;
};

CString GetSplashText(char* item)
{
	CString str = "Unknown";
	switch (item[0])
	{
	case SPLASH_BRIGHTNESS:		
		{
			Str_Brightness* pstruct = (Str_Brightness*) item;
			str.Format("Backlight: %d %c", pstruct->nValue*100/0x14, '%');
		}break;
	case SPLASH_CONTRAST:
		{
			Str_Contrast* pstruct = (Str_Contrast*) item;
			str.Format("Contrast: %d %c", (0xFF-pstruct->nValue)*100/0xFF, '%');
		}break;
	case SPLASH_TEXT:
		{
			Str_Text* pstruct = (Str_Text*) item;
			CString sx = addSlash(pstruct->strText, pstruct->nLength);
			str.Format("Text:[%d:%d] %s", pstruct->nRow+1, pstruct->nCol+1, sx);
		}break;
	case SPLASH_TIMER:
		{
			Str_Timer* pstruct = (Str_Timer*) item;
			if (pstruct->nValue == 65535)
				 str.Format("Timer: forever");
			else str.Format("Timer: %d msec", pstruct->nValue*100);
		}break;
	case SPLASH_COMMAND:
		{
			Str_Command* pstruct = (Str_Command*) item;
			str = "Command: ";

			for (int i=0;i<pstruct->nLen;i++)
			{
				if ( (pstruct->nValue[i] & 0xFE) == 0)
					str += "clear";
				else if ( (pstruct->nValue[i] & 0xFC) == 0)
					str += "return";
				else if ( (pstruct->nValue[i] & 0xF8) == 0)
					str += "entry mode set";
				else if ( (pstruct->nValue[i] & 0xF0) == 0)
					str += "display or cursor on/off";
				else if ( (pstruct->nValue[i] & 0xE0) == 0)
					str += "display or cursor shift";
				else if ( (pstruct->nValue[i] & 0xC0) == 0)
					str += "function set";//should not be used
				else if ( (pstruct->nValue[i] & 0x80) == 0)
					str += "CGRAM";
				else str += "DDRAM";

				if (i!= (pstruct->nLen-1)) str +=", ";
			}
			//str.Format("Command: io=%02x cmd=%02x", pstruct->nIOMap, pstruct->nValue);
		}break;
	case SPLASH_LED:
		{
			Str_Led* pstruct = (Str_Led*) item;
			str = "";
			for (int i=0;i<8;i++)
			{
				if ((1 << i) & pstruct->nValue) str += " X";
				else str += " O";
			}
		}break;
	case SPLASH_FONT:
		{
			Str_Font* pstruct = (Str_Font*) item;
			str.Format("Font: %d", pstruct->nChar);
		}break;
	case SPLASH_REPEAT_START:
		{
			Str_Repeat_Start* pstruct = (Str_Repeat_Start*) item;
			str.Format("Start block (repeat: %d)", pstruct->nRepeatNo);
		}break;
	case SPLASH_REPEAT_STOP:
		{
			Str_Repeat_Stop* pstruct = (Str_Repeat_Stop*) item;
			str.Format("Stop block");
		}break;
	}
	return str;
};

CString removeSlash_str(CString buf)
{
	CString ret = "";

	int cnt_temp = 0;
	while (cnt_temp < buf.GetLength())
	{
		if (buf.GetAt(cnt_temp) != '\\')
		{
			ret += buf.GetAt(cnt_temp);
			cnt_temp ++;
		}
		else
		{
			if ((cnt_temp+1) < buf.GetLength())
			{
				switch (buf.GetAt(cnt_temp+1))
				{
				case '\\': ret += '\\';break;
				case '0':  ret += (char)1;break;
				case '1':  ret += (char)2;break;
				case '2':  ret += (char)3;break;
				case '3':  ret += (char)4;break;
				case '4':  ret += (char)5;break;
				case '5':  ret += (char)6;break;
				case '6':  ret += (char)7;break;
				case '7':  ret += (char)8;break;
				default:
					ret += buf.GetAt(cnt_temp);
					ret += buf.GetAt(cnt_temp+1);
				}
				cnt_temp += 2;
			}
			else
			{
				ret += '\\';
				cnt_temp+=1;
			}
		}
	}

	return ret;
}

void removeSlash_buf(char* buf, int len)
{
	for (int i=0;i<len;i++)
	{
		if (buf[i]<9) buf[i] -= 1;
	}
}

CString addSlash(char* from, int len)
{
	CString str = "";
	for (int i=0;i<len;i++)
	{
		if (from[i]<8) 
		{
			str += '\\';
			str += (char)(from[i]+'0');
		}
		else if (from[i]=='\\')
		{
			str += "\\\\";
		}
		else
		{
			str += from[i];
		}
	}
	return str;
}