// SplashDialog.cpp : implementation file
//

#include "stdafx.h"
#include "SplashEditor.h"
#include "SplashDialog.h"
#include "SplashSelector.h"

#include "PopupText.h"
#include "PopupBrightness.h"
#include "PopupContrast.h"
#include "PopupDelay.h"
#include "PopupCommand.h"
#include "PopupLed.h"
#include "PopupFont.h"
#include "PopupRepeat.h"
#include "PopupBrightnessAutofade.h"
#include "PopupContrastAutofade.h"
#include "PopupTextAutoscroll.h"
#include "PopupLedAutoscroll.h"

#include "..\HEXFile.h"
#include "..\utils\TxFile.h"

#include "..\PicoLCD4x20\PicoLCD4x20Dlg.h"
#include "..\HIDPico4x20.h"

// CSplashDialog dialog

IMPLEMENT_DYNAMIC(CSplashDialog, CDialog)

CSplashDialog::CSplashDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CSplashDialog::IDD, pParent)
	, m_listSplash(true)
	, m_bGenerateHFile(TRUE)
{
	CDialog::Create(IDD_DIALOG_SPLASH, pParent);
	ShowWindow(SW_SHOW);
}

CSplashDialog::~CSplashDialog()
{
	DestroyWindow();
}

void CSplashDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST_SPLASH, m_listSplash);
	DDX_Control(pDX, IDC_STATIC_SP_ACTION, m_sAction);
	DDX_Control(pDX, IDC_PROGRESS_SP, m_pBar);
}


BEGIN_MESSAGE_MAP(CSplashDialog, CDialog)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_DOWNLOAD, &CSplashDialog::OnBnClickedButtonDownload)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_UPLOAD, &CSplashDialog::OnBnClickedButtonUpload)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_APPEND, &CSplashDialog::OnBnClickedButtonAppend)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_DELETE, &CSplashDialog::OnBnClickedButtonDelete)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_INSERT_AFTER, &CSplashDialog::OnBnClickedButtonInsertAfter)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_INSERT_BEFORE, &CSplashDialog::OnBnClickedButtonInsertBefore)
	ON_NOTIFY(NM_DBLCLK, IDC_LIST_SPLASH, &CSplashDialog::OnNMDblclkListSplash)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_MOVE_UP, &CSplashDialog::OnBnClickedSplashButtonMoveUp)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_MOVE_DOWN, &CSplashDialog::OnBnClickedSplashButtonMoveDown)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_ADD_REPEAT, &CSplashDialog::OnBnClickedSplashButtonAddRepeat)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_TEST, &CSplashDialog::OnBnClickedSplashButtonTest)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_CLEAN, &CSplashDialog::OnBnClickedSplashButtonClean)
	ON_BN_CLICKED(IDC_SPLASH_BUTTON_IMPORT, &CSplashDialog::OnBnClickedSplashButtonImport)
END_MESSAGE_MAP()


// CSplashDialog message handlers

BOOL CSplashDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	m_listSplash.Initialize();

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

BOOL CSplashDialog::DestroyWindow()
{
	m_listSplash.RemoveAllMyItems();

	return CDialog::DestroyWindow();
}

int CSplashDialog::GetSplashMaxSize()
{
	return ((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->getSplashMaxSize();
}
int CSplashDialog::GetSplashSize()
{
	return ((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->getSplashSize();
}

bool CSplashDialog::ParseSplashBuffer(unsigned char* data, int& lines)
{
	m_listSplash.RemoveAllMyItems();

	bool ret_ok = true;

	int cnt = 0;
	lines = 0;
	unsigned char* pkg = NULL;
	int init_section_end = -1;

	Str_Command lastCommand;//last text command
	memset(&lastCommand, 0, sizeof(Str_Command));
	
	pkg = data;

	//get max size
	//unsigned short max_len = 0;
	//memcpy(&max_len, pkg, 2);
	//cnt += 2; pkg = data+cnt;

	//get size
	unsigned short len = 0;
	memcpy(&len, pkg, 2);
	cnt += 2; pkg = data+cnt;

	TRACE("Len (buf)=%d\n", len);

	while ((cnt < GetSplashMaxSize()) && (pkg[0]!=0) && (pkg[0]<=PKG_LEN_LCD))
	{
		//TRACE("Len=%02d(%02x) Type=%02x\n",pkg[0], pkg[0], pkg[1]);
		if (init_section_end == -1)
		{//0x03,0x72,0x00,0x00
			if ((pkg[0] == 0x03)&&(pkg[1] == ID_SPLASH_CYCLE_START))
				init_section_end = cnt + (pkg[0]+1);
		}
		else
		{
			
			switch (pkg[1])
			{
			case ID_SPLASH_CYCLE_START:
				{//{0x03,0x72,0x00,0x00};
					Str_Repeat_Start str;
					str.nType	  = SPLASH_REPEAT_START;
					str.nRepeatNo = pkg[3] | (pkg[2] << 8);
					m_listSplash.AddMyItem((char*)&str);
				}break;
			case ID_SPLASH_CYCLE_END:
				{//{0x01,0x73};	
					Str_Repeat_Start str;
					str.nType	  = SPLASH_REPEAT_STOP;
					m_listSplash.AddMyItem((char*)&str);
				}break;
			case ID_SPLASH_TIMER:
				{//{0x03,0x71,0x00,0x0A};
					Str_Timer str;
					str.nType	= SPLASH_TIMER;
					str.nValue	= pkg[3] | (pkg[2] << 8);
					m_listSplash.AddMyItem((char*)&str);
				}break;
			case OUT_REPORT_LED_STATE:
				{//{0x02,0x81,0x00};	
					Str_Led str;
					str.nType  = SPLASH_LED;
					str.nValue = pkg[2];
					m_listSplash.AddMyItem((char*)&str);
				}break;
			case OUT_REPORT_LCD_BACKLIGHT:
				{
					Str_Brightness str;
					str.nType  = SPLASH_BRIGHTNESS;
					str.nValue = pkg[2];
					m_listSplash.AddMyItem((char*)&str);
				}break;
			case OUT_REPORT_LCD_CONTRAST:
				{
					Str_Contrast str;
					str.nType  = SPLASH_CONTRAST;
					str.nValue = pkg[2];
					m_listSplash.AddMyItem((char*)&str);
				}break;
			case OUT_REPORT_CMD:
				{
					//   1,   2,   3,   4,   5,   6,   7,   8,   9
					//0x94,0x00,0x02,0x00,0x64,0x0C,0x00,0x64,0x0C

					Str_Command str;
					str.nType  = SPLASH_COMMAND;
					str.nIOMap = pkg[2];
					str.nLen   = pkg[3];
					if (str.nLen>MAX_CMD_1_LINE) str.nLen=MAX_CMD_1_LINE;
					for (int i=0;i<str.nLen;i++)
					{
						str.nTime[i]  = pkg[i*3+5] | (pkg[i*3+4] << 8);
						str.nValue[i] = pkg[i*3+6];
					}

					if ((str.nLen == 1)&&((str.nValue[0] & 0xC0)!=0))
					{//font or text setting command
						memcpy(&lastCommand, &str, sizeof(Str_Command));
					}
					else m_listSplash.AddMyItem((char*)&str);
				}break;
			case OUT_REPORT_DATA:
				{
					if (lastCommand.nLen > 0)
					{
						if (lastCommand.nValue[0] & 0x80)
						{//text
							Str_Text str;
							str.nType		= SPLASH_TEXT;
							str.nIOMapCmd	= lastCommand.nIOMap;
							str.nTimeCmd	= lastCommand.nTime[0];
							str.nIOMapText	= pkg[2];
							str.nTimeText	= pkg[4] | (pkg[3] << 8);
							
							str.nLength		= pkg[5];
							if (str.nLength	> 20) str.nLength = 20;

							int addr		= lastCommand.nValue[0] & 0x7F;
							str.nRow		= addr / 0x40 + ((addr % 0x40) / 0x14)*2;
							str.nCol		= (addr % 0x40) % 0x14;
							for (int i=0;i<str.nLength;i++)
								str.strText[i] = pkg[i+6];
							str.strText[20] = 0;

							//for (int i=0;i<str.nLength;i++)
							//	if (str.strText[i] < 8) str.strText[i] += 1;

							m_listSplash.AddMyItem((char*)&str);
						}
						else
						{//font
							Str_Font str;
							str.nType		= SPLASH_FONT;
							str.nIOMapFont	= pkg[2];
							str.nTimeFont	= pkg[4] | (pkg[3] << 8);
							str.nIOMapCmd	= lastCommand.nIOMap;
							str.nTimeCmd	= lastCommand.nTime[0];
							int addr		= lastCommand.nValue[0] & 0x7F;
							str.nChar		= (addr - 0x40) / 8;
							for (int i=0;i<pkg[5];i++)
								str.chRows[i] = pkg[i+6];
							m_listSplash.AddMyItem((char*)&str);
						}
					}
				}break;
			default:
				ret_ok = false;
			}
		}

		cnt += (pkg[0]+1);
		lines++;
		pkg = data+cnt;
	}

	TRACE("Len (count)=%d -2\n", cnt);

	if (init_section_end == -1) ret_ok = false;

	return ret_ok;
}

void CSplashDialog::writeHFile(FILE* file, CString text, const unsigned char* data)
{//rom const unsigned char EndOfLCDInitSeq[ 4] = {0x03,0x72,0x00,0x00};
	if (file)
	{
		m_nHCounter ++;

		if (data[1] == ID_SPLASH_CYCLE_START) fprintf(file, "\r\n");

		text = text.Left(12);
		while (text.GetLength()<12) text += "_";

		fprintf(file, "//%x (%d)\r\n", m_nAddrCounter+2, m_nAddrCounter+2);

		fprintf(file, "rom const unsigned char %s_%08d[%02d] = {", text, m_nHCounter, data[0]+1);
		for (int i=0;i<=data[0];i++)
		{
			if (i<data[0])
				 fprintf(file, "0x%02X,", data[i]);
			else fprintf(file, "0x%02X};\r\n", data[i]);
		}

		if (data[1] == ID_SPLASH_CYCLE_END) fprintf(file, "\r\n");

		m_nAddrCounter += (data[0]+1);
	}
}

bool CSplashDialog::BuildSplashBuffer(unsigned char* data, int& real_len, int& comp_len)
{
	UpdateData(TRUE);

	m_nHCounter    = 0;
	m_nAddrCounter = 0;
	FILE* hFile = NULL;
	if (m_bGenerateHFile)
	{
		hFile = fopen("SplashContent.h","wt");
		fprintf(hFile, "rom const unsigned int  SplashSize   =                                              \r\n\r\n");
	}

	real_len = 2;//real len starts from 2 (first two bytes are the length of the splash)
	comp_len = 2;//complete len;

	TxFile file("lcd.ini");
	if (file.Open() != 0)
	{//ini file sequence not found - write internal one
		const unsigned char StartOfLCDInitSeq[ 4] = {0x03,0x71,0x00,0x14};
		memcpy(data+real_len, StartOfLCDInitSeq, sizeof(StartOfLCDInitSeq));
		real_len += sizeof(StartOfLCDInitSeq);
		comp_len += sizeof(StartOfLCDInitSeq);
		writeHFile(hFile, "Init4Code", StartOfLCDInitSeq);
		
		const unsigned char LCDInitSequence[25] = {0x18,0x94,0x00,0x07,0x13,0x88,0x30, 
																		0x00,0xC8,0x30,
																		0x00,0x64,0x30,
																		0x00,0x64,0x38,
																		0x00,0x64,0x08,
																		0x07,0xD0,0x01,
																		0x00,0x64,0x06}; // 4x20 Topway LCD intialization sequence	

		memcpy(data+real_len, LCDInitSequence, sizeof(LCDInitSequence));
		real_len += sizeof(LCDInitSequence);
		comp_len += sizeof(LCDInitSequence);

		writeHFile(hFile, "Init4Code", LCDInitSequence);
	}
	else
	{
		HArray txarray;
		int ret = file.NextLine(&txarray);
		while (ret != -1)
		{
			if (ret == 1)
			{
				if (txarray.length()>0)
				{
					unsigned char length = (unsigned char)txarray.length();

					memcpy(data+real_len  , &length, 1);
					memcpy(data+real_len+1, txarray.getBuf(), length);
					writeHFile(hFile, "Init4File", data+real_len);	
					
					real_len += (length+1);
					comp_len += (length+1);
				}
			}
			ret = file.NextLine(&txarray);
		}
	}

///////// real splash start ////////////
	const unsigned char StartSplash[ 4] = {0x03,0x72,0x00,0x00};
	// Must be inscripted at the end of LCD init sequence - marks the start of the splash
	memcpy(data+real_len, StartSplash, sizeof(StartSplash));
	real_len += sizeof(StartSplash);
	comp_len += sizeof(StartSplash);
	writeHFile(hFile, "SplashStart", StartSplash);

	int temp_len = 0;
	for (int i=0;i<m_listSplash.GetItemCount();i++)
	{
		char* buf = (char*) m_listSplash.GetItemData(i);
		switch (((int*)buf)[0])
		{
		case SPLASH_BRIGHTNESS:
			{
				temp_len = 3;
				if ((comp_len + temp_len) < GetSplashMaxSize())
				{
					Str_Brightness* pold = (Str_Brightness*)buf;
					data[real_len]		= 0x02;
					data[real_len+1]	= OUT_REPORT_LCD_BACKLIGHT;
					data[real_len+2]	= pold->nValue;

					writeHFile(hFile, "Brightness", data + real_len);

					real_len += temp_len;
				}
				comp_len += temp_len;
			}break;
		case SPLASH_CONTRAST:
			{
				temp_len = 3;
				if ((comp_len + temp_len) < GetSplashMaxSize())
				{
					Str_Contrast* pold	= (Str_Contrast*)buf;
					data[real_len]		= 0x02;
					data[real_len+1]	= OUT_REPORT_LCD_CONTRAST;
					data[real_len+2]	= pold->nValue;

					writeHFile(hFile, "Contrast", data + real_len);

					real_len += temp_len;
				}
				comp_len += temp_len;
			}break;
		case SPLASH_TEXT:
			{
				Str_Text* pold = (Str_Text*)buf;
				temp_len = 7 + 6 + pold->nLength;
				if ((comp_len + temp_len) < GetSplashMaxSize())
				{
					//address setting
					data[real_len]		= 6;
					data[real_len+1]	= OUT_REPORT_CMD;
					data[real_len+2]	= pold->nIOMapCmd;
					data[real_len+3]	= 1;
					data[real_len+4]	= (pold->nTimeCmd >> 8) & 0xFF;
					data[real_len+5]	= pold->nTimeCmd & 0xFF;
					int addr = (pold->nRow % 2)*0x40 + (pold->nRow / 2)*0x14 + pold->nCol;
					data[real_len+6]	= addr | 0x80;

					writeHFile(hFile, "TextCommand", data + real_len);

					real_len += 7;
					
					//text
					data[real_len]		= 5+pold->nLength;
					data[real_len+1]	= OUT_REPORT_DATA;
					data[real_len+2]	= pold->nIOMapText;
					data[real_len+3]	= (pold->nTimeText >> 8) & 0xFF;
					data[real_len+4]	= pold->nTimeText & 0xFF;
					data[real_len+5]	= pold->nLength;
					for (int i=0;i<pold->nLength;i++)
						data[real_len+6+i] = pold->strText[i];

					writeHFile(hFile, "TextData", data + real_len);

					real_len += (6 + pold->nLength);
				}
				comp_len += temp_len;
			}break;
		case SPLASH_TIMER:
			{
				temp_len = 4;
				if ((comp_len + temp_len) < GetSplashMaxSize())
				{
					Str_Timer* pold		= (Str_Timer*)buf;
					data[real_len]		= 0x03;
					data[real_len+1]	= ID_SPLASH_TIMER;
					data[real_len+2]	= (pold->nValue >> 8) & 0xFF;
					data[real_len+3]	= pold->nValue & 0xFF;

					writeHFile(hFile, "Timer", data + real_len);

					real_len += temp_len;
				}
				comp_len += temp_len;
			}break;
		case SPLASH_COMMAND:
			{
				Str_Command* pold = (Str_Command*)buf;
				
				temp_len = 4 + 3 * pold->nLen;
				if ((comp_len + temp_len) < GetSplashMaxSize())
				{
					data[real_len]		= 3 + 3 * pold->nLen;
					data[real_len+1]	= OUT_REPORT_CMD;
					data[real_len+2]	= pold->nIOMap;
					data[real_len+3]	= pold->nLen;
					for (int i=0;i<pold->nLen;i++)
					{
						data[real_len+4+i*3] = (pold->nTime[i] >> 8) & 0xFF;
						data[real_len+5+i*3] = pold->nTime[i] & 0xFF;
						data[real_len+6+i*3] = pold->nValue[i];
					}

					writeHFile(hFile, "Command", data + real_len);

					real_len += temp_len;
				}
				comp_len += temp_len;
			}break;
		case SPLASH_LED:
			{
				temp_len = 3;
				if ((comp_len + temp_len) < GetSplashMaxSize())
				{
					Str_Led* pold		= (Str_Led*)buf;
					data[real_len]		= 0x02;
					data[real_len+1]	= OUT_REPORT_LED_STATE;
					data[real_len+2]	= pold->nValue;

					writeHFile(hFile, "LedStatus", data + real_len);

					real_len += temp_len;
				}
				comp_len += temp_len;
			}break;
		case SPLASH_FONT:
			{
				Str_Font* pold = (Str_Font*)buf;
				temp_len = 7 + 14;
				if ((comp_len + temp_len) < GetSplashMaxSize())
				{
					//address setting
					data[real_len]		= 6;
					data[real_len+1]	= OUT_REPORT_CMD;
					data[real_len+2]	= pold->nIOMapCmd;
					data[real_len+3]	= 1;
					data[real_len+4]	= (pold->nTimeCmd >> 8) & 0xFF;
					data[real_len+5]	= pold->nTimeCmd & 0xFF;
					data[real_len+6]	= (pold->nChar*8 + 0x40);// | 0xC0;????

					writeHFile(hFile, "FontCommand", data + real_len);

					real_len += 7;
					
					//font
					data[real_len]		= 13;
					data[real_len+1]	= OUT_REPORT_DATA;
					data[real_len+2]	= pold->nIOMapFont;
					data[real_len+3]	= (pold->nTimeFont >> 8) & 0xFF;
					data[real_len+4]	= pold->nTimeFont & 0xFF;
					data[real_len+5]	= 8;
					for (int i=0;i<8;i++)
						data[real_len+6+i] = pold->chRows[i];

					writeHFile(hFile, "FontData", data + real_len);

					real_len += 14;
				}
				comp_len += temp_len;
			}break;
		case SPLASH_REPEAT_START:
			{
				temp_len = 4;
				if ((comp_len + temp_len) < GetSplashMaxSize())
				{
					Str_Repeat_Start* pold = (Str_Repeat_Start*)buf;
					data[real_len]		= 0x03;
					data[real_len+1]	= ID_SPLASH_CYCLE_START;
					data[real_len+2]	= (pold->nRepeatNo >> 8) & 0xFF;
					data[real_len+3]	= pold->nRepeatNo & 0xFF;

					writeHFile(hFile, "CycleStart", data + real_len);

					real_len += temp_len;
				}
				comp_len += temp_len;
			}break;
		case SPLASH_REPEAT_STOP:
			{
				temp_len = 2;
				if ((comp_len + temp_len) < GetSplashMaxSize())
				{
					Str_Repeat_Stop* pold = (Str_Repeat_Stop*)buf;
					data[real_len]		= 0x01;
					data[real_len+1]	= ID_SPLASH_CYCLE_END;

					writeHFile(hFile, "CycleStop", data + real_len);

					real_len += temp_len;
				}
				comp_len += temp_len;
			}break;
		}
	}

	if ((comp_len + 1) < GetSplashMaxSize())
	{
		data[real_len]	= 0xFF;//end of splash
		real_len += 1;
	}
	comp_len += 1;

	if (hFile)
	{
		rewind(hFile);
		fprintf(hFile, "rom const unsigned int  SplashSize   = %d;", real_len-3);
		
		fclose(hFile);
	}

	return (comp_len < GetSplashMaxSize());
}

void CSplashDialog::OnBnClickedButtonDownload()
{
	if (m_listSplash.GetItemCount() != 0)
	{
		if (AfxMessageBox("Are you sure you want to overwrite the edited content?",MB_OKCANCEL) != IDOK)
			return;
	}

	CWaitCursor cursor;

	if (((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->readFile(false, TYPE_CODE_SPLASH, "splash_read.hex", 0, GetSplashSize()+3))
	{
		unsigned char* data = new unsigned char[GetSplashSize()+3];
		
		if (HIDPico4x20::loadFile2Array(data, GetSplashSize()+3, "splash_read.hex"))
		{
			int lines = 0;
			bool ok = ParseSplashBuffer(data, lines);
			TRACE("Lines1 = %d\n", lines);
			if (ok)
				m_sAction.SetWindowText("Splash uploaded!");
			else 
				m_sAction.SetWindowText("Splash uploaded with errors!!!");
		}
		else m_sAction.SetWindowText("Parsing not succesfull! (1)");
		
		delete data;
	}
	else m_sAction.SetWindowText("Upload failed");
}

void CSplashDialog::OnBnClickedButtonUpload()
{
	if (m_listSplash.GetItemCount() == 0)
	{
		if (AfxMessageBox("Are you sure you want to upload empty content?",MB_OKCANCEL) != IDOK)
			return;
	}

	unsigned char* data = new unsigned char[GetSplashMaxSize()+64];
	int real_len = 2;
	int comp_len = 2;
	bool ok = BuildSplashBuffer(data, real_len, comp_len);
	
	CString str;
	str.Format("Splash uses %d from %d bytes",real_len, GetSplashMaxSize());
	
	unsigned short write_len = (unsigned short)real_len - 3;
	memcpy(data, &write_len, 2);

	MyTrace(1, "WRITe LENGTH=%d\n", write_len);

	if (ok)
	{
		if (((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->writeFile4Array(32, "splash_write.hex", data, real_len))
		{
			if (((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->writeFile(false, TYPE_CODE_SPLASH, "splash_write.hex", -1))
			{
				str = "Splash uploaded successfully!";
				AfxMessageBox(str);
			}
			else 
			{
				str = "ERROR: Could not upload splash to LCD!";
				AfxMessageBox(str);
			}

			((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->getMaxSplash();
		}
		else 
		{
			str = "ERROR: Could not write temporary splash file!";
			AfxMessageBox(str);
		}
	}
	else
	{
		AfxMessageBox("Splash data too long!");
		str = "ERROR: "+ str;
	}

	m_sAction.SetWindowText(str);
	
	delete data;
}

bool CSplashDialog::ReadOneSplashItem(char* buf, int insert_type)
{
	bool new_splash = false;
	int sel_type;

	if (((int*)buf)[0] == -1)
	{
		CSplashSelector dlg(this);
		if (dlg.DoModal() == IDOK)
			 sel_type = dlg.m_nSelectedType;
		else return false;

		new_splash = true;
	}
	else sel_type = ((int*)buf)[0];

	switch (sel_type)
	{
	case SPLASH_BRIGHTNESS:
		{
			Str_Brightness* pold = (Str_Brightness*)buf;

			CPopupBrightness dlg(this);
			if (new_splash)
				 dlg.m_nBrightness = 0x14;
			else dlg.m_nBrightness = pold->nValue;

			if (dlg.DoModal() == IDOK)
			{
				Str_Brightness str;
				str.nType  = SPLASH_BRIGHTNESS;
				str.nValue = dlg.m_nBrightness;
				memcpy(buf, (char*)&str, sizeof(str));
			}
			else return false;
		}break;
	case SPLASH_CONTRAST:
		{
			Str_Contrast* pold = (Str_Contrast*)buf;

			CPopupContrast dlg(this);
			if (new_splash)
				 dlg.m_nContrast = 0xFF;
			else dlg.m_nContrast = 0xFF - pold->nValue;

			if (dlg.DoModal() == IDOK)
			{
				Str_Contrast str;
				str.nType  = SPLASH_CONTRAST;
				str.nValue = 0xFF - dlg.m_nContrast;
				memcpy(buf, (char*)&str, sizeof(str));
			}
			else return false;
		}break;
	case SPLASH_TEXT:
		{
			Str_Text* pold = (Str_Text*)buf;

			CPopupText dlg(this);
			if (new_splash)
			{
				dlg.m_nRow		= 1;
				dlg.m_nColumn	= 1;
				dlg.m_strText	= "";
			}
			else
			{
				dlg.m_nRow		= pold->nRow + 1;
				dlg.m_nColumn	= pold->nCol + 1;
				dlg.m_strText	= addSlash(pold->strText, pold->nLength);
			}

			if (dlg.DoModal() == IDOK)
			{
				Str_Text str;
				str.nType		= SPLASH_TEXT;
				str.nRow		= dlg.m_nRow	- 1;
				str.nCol		= dlg.m_nColumn	- 1;
				str.nIOMapText	= 0x01;
				str.nTimeText	= 0x40;
				str.nIOMapCmd	= 0x00;
				str.nTimeCmd	= 0x64;

				dlg.m_strText = removeSlash_str(dlg.m_strText);
								
				if (dlg.m_strText.GetLength()>(20-str.nCol))
					dlg.m_strText = dlg.m_strText.Left(20-str.nCol-1);
				else while (dlg.m_strText.GetLength() < (20-str.nCol))
					dlg.m_strText += " ";
				
				strcpy(str.strText, dlg.m_strText);
				removeSlash_buf(str.strText, 20);
				str.nLength = strlen(dlg.m_strText);

				memcpy(buf, (char*)&str, sizeof(str));
			}
			else return false;
		}break;
	case SPLASH_TIMER:
		{
			Str_Timer* pold = (Str_Timer*)buf;

			CPopupDelay dlg(this);
			if (new_splash)
				 dlg.m_nDelay = 10;
			else dlg.m_nDelay = pold->nValue;

			if (dlg.DoModal() == IDOK)
			{
				Str_Timer str;
				str.nType  = SPLASH_TIMER;
				str.nValue = dlg.m_nDelay;
				memcpy(buf, (char*)&str, sizeof(str));
			}
			else return false;
		}break;
	case SPLASH_COMMAND:
		{
			Str_Command* pold = (Str_Command*)buf;
			CPopupCommand dlg(this);
			if (!new_splash) dlg.ParseCommand(pold);

			if (dlg.DoModal() == IDOK)
			{
				Str_Command str;
				str.nType  = SPLASH_COMMAND;
				str.nIOMap = 0x00;

				dlg.RetrieveCommand(&str);
				
				memcpy(buf, (char*)&str, sizeof(str));
			}
			else return false;
		}break;
	case SPLASH_LED:
		{
			Str_Led* pold = (Str_Led*)buf;
			CPopupLed dlg(this);
			if (!new_splash) dlg.ParseLed(pold);

			if (dlg.DoModal() == IDOK)
			{
				Str_Led str;
				str.nType  = SPLASH_LED;

				dlg.RetrieveLed(&str);
				
				memcpy(buf, (char*)&str, sizeof(str));
			}
			else return false;
		}break;
	case SPLASH_FONT:
		{
			Str_Font* pold = (Str_Font*)buf;
			CPopupFont dlg(this);
			if (!new_splash) dlg.ParseFont(pold);

			if (dlg.DoModal() == IDOK)
			{
				Str_Font str;
				str.nType		= SPLASH_FONT;
				str.nIOMapFont	= 0x01;
				str.nTimeFont	= 0x40;
				str.nIOMapCmd	= 0x00;
				str.nTimeCmd	= 0x64;

				dlg.RetrieveFont(&str);
				
				memcpy(buf, (char*)&str, sizeof(str));
			}
			else return false;
		}break;
	case SPLASH_REPEAT_START:
		{
			if (new_splash) return false;
			Str_Repeat_Start* pold = (Str_Repeat_Start*)buf;

			CPopupRepeat dlg(this);
			dlg.m_nRepeat = pold->nRepeatNo;
			if (dlg.DoModal() == IDOK)
			{
				Str_Repeat_Start str;
				str.nType  = SPLASH_REPEAT_START;
				str.nRepeatNo = dlg.m_nRepeat;
				memcpy(buf, (char*)&str, sizeof(str));
			}
			else return false;
		}break;
	case SPLASH_BRIGHTNESS_AUTOFADING:
		{
			if (!new_splash) return false;

			CPopupBrightnessAutofade dlg(this);
			if (dlg.DoModal() == IDOK)
			{
				if (dlg.m_nBrightness1 < dlg.m_nBrightness2)
				{
					float cnt  = dlg.m_nBrightness1;
					float step = ((float)dlg.m_nBrightness2-(float)dlg.m_nBrightness1)/(float)dlg.m_nSteps;

					int row_pos = 0;
					while (cnt<=(float)dlg.m_nBrightness2)
					{
						Str_Brightness str;
						str.nType  = SPLASH_BRIGHTNESS;
						str.nValue = cnt;
						
						AddAutomaticItem((char*)&str, insert_type, row_pos);

						Str_Timer str2;
						str2.nType  = SPLASH_TIMER;
						str2.nValue = dlg.m_nDelay;
						
						AddAutomaticItem((char*)&str2, insert_type, row_pos);

						cnt += step;
					}	
				}
				else if (dlg.m_nBrightness1 > dlg.m_nBrightness2)
				{
					float cnt  = dlg.m_nBrightness1;
					float step = ((float)dlg.m_nBrightness1-(float)dlg.m_nBrightness2)/(float)dlg.m_nSteps;

					int row_pos = 0;
					while (cnt>=(float)dlg.m_nBrightness2)
					{
						Str_Brightness str;
						str.nType  = SPLASH_BRIGHTNESS;
						str.nValue = cnt;
						
						AddAutomaticItem((char*)&str, insert_type, row_pos);

						Str_Timer str2;
						str2.nType  = SPLASH_TIMER;
						str2.nValue = dlg.m_nDelay;
						
						AddAutomaticItem((char*)&str2, insert_type, row_pos);

						cnt -= step;
					}	
				}
			}
		}break;
	case SPLASH_CONTRAST_AUTOFADING:
		{
			if (!new_splash) return false;

			CPopupContrastAutofade dlg(this);
			if (dlg.DoModal() == IDOK)
			{
				dlg.m_nContrast1 = 0xFF - dlg.m_nContrast1;
				dlg.m_nContrast2 = 0xFF - dlg.m_nContrast2;

				if (dlg.m_nContrast1 < dlg.m_nContrast2)
				{
					float cnt  = dlg.m_nContrast1;
					float step = ((float)dlg.m_nContrast2-(float)dlg.m_nContrast1)/(float)dlg.m_nSteps;

					int row_pos = 0;
					while (cnt<=(float)dlg.m_nContrast2)
					{
						Str_Contrast str;
						str.nType  = SPLASH_CONTRAST;
						str.nValue = cnt;
						AddAutomaticItem((char*)&str, insert_type, row_pos);

						Str_Timer str2;
						str2.nType  = SPLASH_TIMER;
						str2.nValue = dlg.m_nDelay;
						AddAutomaticItem((char*)&str2, insert_type, row_pos);

						cnt += step;
					}	
				}
				else if (dlg.m_nContrast1 > dlg.m_nContrast2)
				{
					float cnt  = dlg.m_nContrast1;
					float step = ((float)dlg.m_nContrast1-(float)dlg.m_nContrast2)/(float)dlg.m_nSteps;

					int row_pos = 0;
					while (cnt>=(float)dlg.m_nContrast2)
					{
						Str_Contrast str;
						str.nType  = SPLASH_CONTRAST;
						str.nValue = cnt;
						AddAutomaticItem((char*)&str, insert_type, row_pos);

						Str_Timer str2;
						str2.nType  = SPLASH_TIMER;
						str2.nValue = dlg.m_nDelay;
						AddAutomaticItem((char*)&str2, insert_type, row_pos);

						cnt -= step;
					}	
				}
			}
		}break;
	case SPLASH_TEXT_AUTOSCROLL:
		{
			if (!new_splash) return false;

			CPopupTextAutoscroll dlg(this);
			if (dlg.DoModal() == IDOK)
			{
				if (dlg.m_strScrollText.GetLength()>20)
						dlg.m_strScrollText = dlg.m_strScrollText.Left(20-1);
				else while (dlg.m_strScrollText.GetLength() < 20)
						dlg.m_strScrollText += " ";

				int row_pos = 0;
				for (int cnt = 0;cnt < dlg.m_nScrollTimes; cnt++)
				{
					Str_Text str;
					str.nType		= SPLASH_TEXT;
					str.nRow		= dlg.m_nRow	- 1;
					str.nCol		= 0;
					str.nIOMapText	= 0x01;
					str.nTimeText	= 0x40;
					str.nIOMapCmd	= 0x00;
					str.nTimeCmd	= 0x64;

					strcpy(str.strText, dlg.m_strScrollText);
					str.nLength = strlen(dlg.m_strScrollText);

					AddAutomaticItem((char*)&str, insert_type, row_pos);
				
					Str_Timer str2;
					str2.nType  = SPLASH_TIMER;
					str2.nValue = dlg.m_nScrollDelay;

					AddAutomaticItem((char*)&str2, insert_type, row_pos);

					if (dlg.m_bScrollLeft)
						 dlg.m_strScrollText = dlg.m_strScrollText.Right(19) + " ";
					else dlg.m_strScrollText = " "+dlg.m_strScrollText.Left(19);
				}	
			}
		}break;
	case SPLASH_LED_AUTOSCROLL:
		{
			if (!new_splash) return false;

			CPopupLedAutoscroll dlg(this);
			if (dlg.DoModal() == IDOK)
			{
				int row_pos = 0;
				for (int cnt = 0;cnt < dlg.m_nScrollTimes; cnt++)
				{
					Str_Led str;
					str.nType  = SPLASH_LED;

					dlg.RetrieveLed(&str, cnt);

					AddAutomaticItem((char*)&str, insert_type, row_pos);
				
					Str_Timer str2;
					str2.nType  = SPLASH_TIMER;
					str2.nValue = dlg.m_nScrollDelay;
					AddAutomaticItem((char*)&str2, insert_type, row_pos);
				}	
			}
		}break;
	default:
		return false;
	}
	return (((int*)buf)[0]!=-1);
}

void CSplashDialog::AddAutomaticItem(char* item, int insert_type, int& row_pos)
{
	if (insert_type == -1)
	{
		POSITION pos = m_listSplash.GetFirstSelectedItemPosition();
		if (pos != NULL)
		{
			int row = m_listSplash.GetNextSelectedItem(pos);
			m_listSplash.InsertMyItem(row, item);
		}
	}
	else if (insert_type == +1)
	{
		POSITION pos = m_listSplash.GetFirstSelectedItemPosition();
		if (pos != NULL)
		{
			int row = m_listSplash.GetNextSelectedItem(pos);
			m_listSplash.InsertMyItem(row+1+row_pos, item);

			row_pos++;
		}
	}
	else m_listSplash.AddMyItem(item);
}

void CSplashDialog::OnBnClickedButtonAppend()
{
	char buf[SPLASH_ELEMENT_MAX_SIZE];
	
	((int*)buf)[0]  = -1;
	if (ReadOneSplashItem(buf, 0))
		m_listSplash.AddMyItem(buf);
}

void CSplashDialog::OnBnClickedButtonDelete()
{
	POSITION pos = m_listSplash.GetFirstSelectedItemPosition();
	if (pos != NULL)
	{
		if (AfxMessageBox("Are you sure?",MB_OKCANCEL) == IDOK)
		{
			int row = m_listSplash.GetNextSelectedItem(pos);
			int cnt = m_listSplash.GetSelectedCount();

			if (cnt>1)
			{
				int start_blocks = 0;
				int end_blocks = 0;
				for (int i=0;i<cnt;i++)
				{
					int* data  = (int*) m_listSplash.GetItemData(row+i);
					if (data[0] == SPLASH_REPEAT_START)
						start_blocks++;
					else if (data[0] == SPLASH_REPEAT_STOP)
					{
						end_blocks ++;
						if (start_blocks == 0) break;//first repeat is an end block which means unfinished repeat
					}
				}
				if (start_blocks!=end_blocks)
				{
					AfxMessageBox("Cannot delete different amount of repeat start blocks and end blocks!");
					return;
				}
				for (int i=0;i<cnt;i++)
					m_listSplash.DeleteMyItem(row);
			}
			else
			{
				int* data  = (int*) m_listSplash.GetItemData(row);
				int type = data[0];
				
				if (type == SPLASH_REPEAT_START)
				{//search below for the first end block and delete it
					int i = row+1;
					int found = -1;
					int starts = 1;
					while ((i<m_listSplash.GetItemCount())&&(found==-1))
					{
						int* data2  = (int*) m_listSplash.GetItemData(i);
						if (data2[0] == SPLASH_REPEAT_STOP)
						{
							starts--;
							if (starts == 0) 
								found = i;
						}
						else if (data2[0] == SPLASH_REPEAT_START)
							starts++;
						
						i++;
					}

					if (found == -1)
					{
						AfxMessageBox("ERROR: repeat block cannot be deleted (end of block not found)!");
					}
					else
					{
						m_listSplash.DeleteMyItem(found);
						m_listSplash.DeleteMyItem(row);
					}
				}
				else if (type == SPLASH_REPEAT_STOP)
				{
					int i = row-1;
					int found = -1;
					int stops = 1;
					while ((i>=0)&&(found==-1))
					{
						int* data2  = (int*) m_listSplash.GetItemData(i);
						if (data2[0] == SPLASH_REPEAT_START)
						{
							stops --;
							if (stops == 0)
								found = i;
						}
						else if (data2[0] == SPLASH_REPEAT_STOP)
							stops++;

						i--;
					}

					if (found == -1)
					{
						AfxMessageBox("ERROR: repeat block cannot be deleted (start of block not found)!");
					}
					else
					{
						m_listSplash.DeleteMyItem(row);
						m_listSplash.DeleteMyItem(found);
					}
				}
				else m_listSplash.DeleteMyItem(row);
			}
		}
	}
}

void CSplashDialog::OnBnClickedButtonInsertAfter()
{
	POSITION pos = m_listSplash.GetFirstSelectedItemPosition();
	if (pos != NULL)
	{
		int row = m_listSplash.GetNextSelectedItem(pos);
	
		char buf[SPLASH_ELEMENT_MAX_SIZE];
		((int*)buf)[0]  = -1;
		if (ReadOneSplashItem(buf, +1))
			m_listSplash.InsertMyItem(row+1, buf);
	}
}

void CSplashDialog::OnBnClickedButtonInsertBefore()
{
	POSITION pos = m_listSplash.GetFirstSelectedItemPosition();
	if (pos != NULL)
	{
		int row = m_listSplash.GetNextSelectedItem(pos);
	
		char buf[SPLASH_ELEMENT_MAX_SIZE];
		((int*)buf)[0]  = -1;
		if (ReadOneSplashItem(buf, -1))
			m_listSplash.InsertMyItem(row, buf);
	}
}

void CSplashDialog::OnNMDblclkListSplash(NMHDR *pNMHDR, LRESULT *pResult)
{
	POSITION pos = m_listSplash.GetFirstSelectedItemPosition();
	if (pos != NULL)
	{
		int row = m_listSplash.GetNextSelectedItem(pos);
		char* data = (char*) m_listSplash.GetItemData(row);
		if (data)
		{
			if (ReadOneSplashItem(data, 0))
				m_listSplash.UpdateMyItemText(row);//data is updated anyway
		}
	}

	*pResult = 0;
}

bool CSplashDialog::CheckNestedCycles()
{
	int opened_cycles = 0;

	for (int i=0;i<m_listSplash.GetItemCount();i++)
	{
		int* data  = (int*) m_listSplash.GetItemData(i);
		if (data[0] == SPLASH_REPEAT_START)
		{
			opened_cycles++;
			if (opened_cycles > MAX_REPEAT_BLOCKS) return false;
		}
		else if (data[0] == SPLASH_REPEAT_STOP)
		{
			opened_cycles--;
			if (opened_cycles < 0) return false;
		}
	}
	
	return true;
}

void CSplashDialog::OnBnClickedSplashButtonMoveUp()
{
	MoveUp(true);
}

void CSplashDialog::MoveUp(bool button)
{
	POSITION pos = m_listSplash.GetFirstSelectedItemPosition();
	if (pos != NULL)
	{
		int row = m_listSplash.GetNextSelectedItem(pos);
		int cnt = m_listSplash.GetSelectedCount();

		row --;
		if (row >= 0)
		{
			int type_tomove = ((int*)m_listSplash.GetItemData(row))[0];

			int first_type = -1;
			int temp_type;
			int opened_blocks = 0;
			for (int i=0;i<cnt;i++)
			{
				temp_type = ((int*)m_listSplash.GetItemData(i+row+1))[0];
				if (temp_type == SPLASH_REPEAT_START)
				{
					opened_blocks ++;
					if (first_type == -1) first_type = temp_type; 
				}
				else if (temp_type == SPLASH_REPEAT_STOP)
				{
					opened_blocks --;
					if (first_type == -1) first_type = temp_type; 
				}
			}

			if ((type_tomove == SPLASH_REPEAT_START)||(type_tomove == SPLASH_REPEAT_STOP))
			{
				if (first_type == SPLASH_REPEAT_STOP) 
				{
					AfxMessageBox("Cannot intercalate (mix) repeat blocks!");
					return;
				}
				else if (first_type == SPLASH_REPEAT_START) 
				{
					if (opened_blocks != 0)
					{
						AfxMessageBox("Cannot intercalate (mix) repeat blocks!");
						return;
					}
				}
			}

			char* data  = (char*) m_listSplash.GetItemData(row);
			char dcopy[SPLASH_ELEMENT_MAX_SIZE];
			if (data)
			{
				memcpy(dcopy, data, GetSplashTypeLen(((int*)data)[0]));
				m_listSplash.DeleteMyItem(row);
				m_listSplash.InsertMyItem(row+cnt, dcopy);

				if (!CheckNestedCycles())
				{
					if (button) MoveDown(false);

					CString str;
					str.Format("Cannot have more than %d nested repeat blocks!",MAX_REPEAT_BLOCKS);
					AfxMessageBox(str);
				}
			}
		}
	}
}

void CSplashDialog::OnBnClickedSplashButtonMoveDown()
{
	MoveDown(true);
}

void CSplashDialog::MoveDown(bool button)
{
	POSITION pos = m_listSplash.GetFirstSelectedItemPosition();
	if (pos != NULL)
	{
		int row = m_listSplash.GetNextSelectedItem(pos);
		int cnt = m_listSplash.GetSelectedCount();

		if ((row + cnt)< m_listSplash.GetItemCount())
		{
			int type_tomove = ((int*)m_listSplash.GetItemData(row+cnt))[0];

			int first_type = -1;
			int temp_type;
			int opened_blocks = 0;
			for (int i=cnt-1;i>=0;i--)
			{
				temp_type = ((int*)m_listSplash.GetItemData(i+row))[0];
				if (temp_type == SPLASH_REPEAT_START)
				{
					opened_blocks --;
					if (first_type == -1) first_type = temp_type; 
				}
				else if (temp_type == SPLASH_REPEAT_STOP)
				{
					opened_blocks ++;
					if (first_type == -1) first_type = temp_type; 
				}
			}

			if ((type_tomove == SPLASH_REPEAT_START)||(type_tomove == SPLASH_REPEAT_STOP))
			{
				if (first_type == SPLASH_REPEAT_START) 
				{
					AfxMessageBox("Cannot intercalate (mix) repeat blocks!");
					return;
				}
				else if (first_type == SPLASH_REPEAT_STOP) 
				{
					if (opened_blocks != 0)
					{
						AfxMessageBox("Cannot intercalate (mix) repeat blocks!");
						return;
					}
				}
			}

			char* data  = (char*) m_listSplash.GetItemData(row+cnt);
			char dcopy[SPLASH_ELEMENT_MAX_SIZE];
			if (data)
			{
				memcpy(dcopy, data, GetSplashTypeLen(((int*)data)[0]));
				m_listSplash.DeleteMyItem(row+cnt);
				m_listSplash.InsertMyItem(row, dcopy);

				if (!CheckNestedCycles())
				{
					if (button) MoveUp(false);

					CString str;
					str.Format("Cannot have more than %d nested repeat blocks!",MAX_REPEAT_BLOCKS);
					AfxMessageBox(str);
				}
			}
		}
	}
}

void CSplashDialog::OnBnClickedSplashButtonAddRepeat()
{
	POSITION pos = m_listSplash.GetFirstSelectedItemPosition();
	if (pos != NULL)
	{
		int row = m_listSplash.GetNextSelectedItem(pos);
		int cnt = m_listSplash.GetSelectedCount();

		int start_blocks = 0;
		int end_blocks = 0;
		for (int i=0;i<cnt;i++)
		{
			int* data  = (int*) m_listSplash.GetItemData(row+i);
			if (data[0] == SPLASH_REPEAT_START)
				start_blocks++;
			else if (data[0] == SPLASH_REPEAT_STOP)
			{
				end_blocks ++;
				if (start_blocks == 0) break;//first repeat is an end block which means unfinished repeat
			}
		}
		if (start_blocks!=end_blocks)
		{
			AfxMessageBox("Cannot intercalate (mix) repeat blocks!");
			return;
		}

		char buf[SPLASH_ELEMENT_MAX_SIZE];
		Str_Repeat_Start* str = (Str_Repeat_Start*) buf;
		str->nType		= SPLASH_REPEAT_START;
		str->nRepeatNo	= 0;
		if (ReadOneSplashItem(buf, 0))
		{
			int row_start = m_listSplash.InsertMyItem(row, buf);
			Str_Repeat_Stop* str = (Str_Repeat_Stop*) buf;
			str->nType		= SPLASH_REPEAT_STOP;
			int row_stop = m_listSplash.InsertMyItem(row+cnt+1, buf);

			if (!CheckNestedCycles())
			{
				m_listSplash.DeleteMyItem(row_stop);
				m_listSplash.DeleteMyItem(row_start);

				CString str;
				str.Format("Cannot add more than %d nested repeat blocks!",MAX_REPEAT_BLOCKS);
				AfxMessageBox(str);
			}
		}
	}
}

void CSplashDialog::OnBnClickedSplashButtonTest()
{
	((CPicoLCD4x20Dlg*)AfxGetMainWnd())->m_pHIDInterfaceLCD->testSplash();
}

void CSplashDialog::OnBnClickedSplashButtonClean()
{
	if (m_listSplash.GetItemCount() != 0)
	{
		if (AfxMessageBox("Are you sure you want to delete the splash content?",MB_OKCANCEL) != IDOK)
			return;

		m_listSplash.RemoveAllMyItems();
	}
}

void CSplashDialog::OnBnClickedSplashButtonImport()
{
	if (m_listSplash.GetItemCount() != 0)
	{
		if (AfxMessageBox("Are you sure you want to overwrite the edited content?",MB_OKCANCEL) != IDOK)
			return;
	}

	char BASED_CODE szFilter[] = "Hex files (*.hex)|*.hex|";
	CFileDialog cfd(TRUE,NULL,"*.hex", OFN_OVERWRITEPROMPT | OFN_EXPLORER,  szFilter,this);
	if (cfd.DoModal()==IDOK)
	{
		CString file = cfd.GetPathName();

		HEXFile hfile(file);
		if (hfile.Open() == 0)
		{
			int length_statistics = 0;
			int ret = hfile.NextLine();
			int start_statistics = hfile.GetGlobalAddress();
			while (ret == 0)
			{
				length_statistics = hfile.GetGlobalAddress()+hfile.GetLength();
				ret = hfile.NextLine();
			}

			if ((length_statistics > 0)&&(length_statistics < 500000))
			{
				unsigned char* data = new unsigned char[length_statistics+1];

				hfile.Reset();
				ret = hfile.NextLine();
				int len;
				int cnt = 0;
				while (ret==0)
				{
					len  = hfile.GetLength();
					memcpy(data+cnt, hfile.GetData(), len);
					cnt += len;
					ret = hfile.NextLine();
				}

				data[cnt] = 0xFF;//needed to stop the parser
				cnt += 1;

				//cnt contains the length of the data what should be parsed
				CWaitCursor cursor;
				int lines = 0;
				bool ok = ParseSplashBuffer(data, lines);

				delete data;

				if ((!ok)||(m_listSplash.GetItemCount()==0))
				{
					AfxMessageBox("Hex file seems to be invalid!");
				}
			}
		}
	}
}
